# Copyright 2014 - Rackspace Hosting
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Solum Deployer Heat handler."""

# import json
import logging
import socket
import time

from heatclient import exc
import httplib2
# from keystoneclient.v2_0 import client as ksclient
from oslo_config import cfg
from oslo_log import log as os_logging
from sqlalchemy import exc as sqla_exc
from swiftclient import exceptions as swiftexp
import yaml

import solum
from solum.api.handlers import userlog_handler
from solum.common import catalog
from solum.common import clients
from solum.common import exception
from solum.common import heat_utils
from solum.common import repo_utils
from solum.common import solum_glanceclient
from solum.common import solum_swiftclient
# from solum.common import utils
from solum import objects
from solum.objects import assembly
from solum.uploaders import tenant_logger as tlog


LOG = os_logging.getLogger(__name__)

STATES = assembly.States


SERVICE_OPTS = [
    cfg.IntOpt('max_attempts',
               default=600,
               help=('Number of attempts to query the Heat stack for '
                     'finding out the status of the created stack and '
                     'getting url of the DU created in the stack')),
    cfg.IntOpt('du_attempts',
               default=500,
               help=('Number of attempts to query the Docker DU for '
                     'finding out the status of the created app and '
                     'getting url of the DU created in the stack')),
    cfg.IntOpt('wait_interval',
               default=1,
               help=('Sleep time interval between two attempts of querying '
                     'the Heat stack. This interval is in seconds.')),
    cfg.FloatOpt('growth_factor',
                 default=1.1,
                 help=('Factor by which sleep time interval increases. '
                       'This value should be >= 1.0')),
    cfg.StrOpt('flavor',
               default="m1.small",
               help='VM Flavor'),
    cfg.StrOpt('image',
               default="coreos",
               help='Image id'),
    cfg.StrOpt('key_name',
               default="mykey",
               help='keypair name'),
    cfg.StrOpt('deployer_log_dir',
               default="/var/log/solum/deployer",
               help='Deployer logs location'),
]


def list_opts():
    yield 'deployer', SERVICE_OPTS


cfg.CONF.register_opts(SERVICE_OPTS, group='deployer')
cfg.CONF.import_opt('image_format', 'solum.api.handlers.assembly_handler',
                    group='api')
cfg.CONF.import_opt('image_storage', 'solum.worker.config', group='worker')
cfg.CONF.import_opt('www_authenticate_uri', 'keystonemiddleware.auth_token',
                    group='keystone_authtoken')


deployer_log_dir = cfg.CONF.deployer.deployer_log_dir


def get_heat_client(ctxt, app):
    # raw_content = json.loads(app.raw_content)
    # username = raw_content['username']
    # encoded_password = raw_content['password'].encode('ISO-8859-1')
    # decrypted_password = utils.decrypt(encoded_password)
    # password = decrypted_password
    # tenant_name = raw_content['tenant_name']
    # auth_url = cfg.CONF.keystone_authtoken.www_authenticate_uri

    # ks_kwargs = {
    #     'username': username,
    #     'password': password,
    #     'tenant_name': tenant_name,
    #     'auth_url': auth_url
    # }

    # k_client = ksclient.Client(**ks_kwargs)
    # auth_token = k_client.auth_token

    osc = clients.OpenStackClients(ctxt)

    # TODO(zhurong): Following works for github triggers.
    # We should accommodate it with the current workflow.
    # See for details: https://bugs.launchpad.net/solum/+bug/1671871
    # heat = osc.heat(username, password, auth_token)

    heat = osc.heat(token=ctxt.auth_token)

    return heat


def save_du_ref_for_scaling(ctxt, assembly_id, du=None):

    try:
        wf = objects.registry.Workflow.get_by_assembly_id(assembly_id)
    except sqla_exc.SQLAlchemyError as ex:
        LOG.error("Failed to get workflow corresponding "
                  "to assembly %s" % assembly_id)
        LOG.exception(ex)
        return

    if wf is not None:
        try:
            app = objects.registry.App.get_by_id(ctxt, wf.app_id)
            current_scale_config = app.scale_config
            if current_scale_config:
                current_config = current_scale_config[app.name]
                current_config['du'] = du
                current_scale_config[app.name] = current_config
                scale_config = dict()
                scale_config['scale_config'] = current_scale_config
                objects.registry.App.update_and_save(ctxt, app.id,
                                                     scale_config)
        except sqla_exc.SQLAlchemyError as ex:
            LOG.error("Failed to update app scale_config: %s" % app.id)
            LOG.exception(ex)


def get_assembly_by_id(ctxt, assembly_id):
    return solum.objects.registry.Assembly.get_by_id(ctxt, assembly_id)


def get_app_by_assem_id(ctxt, assembly_id):
    assem = get_assembly_by_id(ctxt, assembly_id)
    if assem:
        plan = solum.objects.registry.Plan.get_by_id(ctxt, assem.plan_id)
        app = solum.objects.registry.App.get_by_id(ctxt, plan.uuid)
        return app


def update_wf_and_app(ctxt, assembly_id, data):
    # Update workflow and app objects
    data_dict = dict()
    if data.get('status') is not None:
        data_dict['status'] = data['status']
    if data.get('application_uri') is not None:
        data_dict['app_url'] = data['application_uri']

    wf = None
    try:
        wf = objects.registry.Workflow.get_by_assembly_id(assembly_id)
        objects.registry.Workflow.update_and_save(ctxt, wf.id, data_dict)
    except sqla_exc.SQLAlchemyError as ex:
        LOG.error("Failed to update workflow corresponding to assembly %s"
                  % assembly_id)
        LOG.exception(ex)
    except exception.ResourceNotFound as ex:
        #  This happens  if plan (deprecated) was directly created
        LOG.error("Workflow not found for assembly %s" % assembly_id)
        LOG.exception(ex)

    if wf is not None:
        try:
            app = objects.registry.App.get_by_id(ctxt, wf.app_id)
            objects.registry.App.update_and_save(ctxt, app.id, data_dict)
        except sqla_exc.SQLAlchemyError as ex:
            LOG.error("Failed to update app status and app URL: %s" % app.id)
            LOG.exception(ex)


def update_assembly(ctxt, assembly_id, data):
    # Here we are updating the assembly synchronously (i.e. without
    # using the conductor). This is because when using the conductor latency
    # is introduced between the update call and when assembly's state is
    # actually updated in the database. This latency leads to concurrency
    # bugs within deployers' actions when multiple deployers are present
    # in the system.
    try:
        objects.registry.Assembly.update_and_save(ctxt, assembly_id, data)
    except sqla_exc.SQLAlchemyError as ex:
        LOG.error("Failed to update assembly status, ID: %s" % assembly_id)
        LOG.exception(ex)

    try:
        update_wf_and_app(ctxt, assembly_id, data)
    except Exception as ex:
        LOG.error("Failed to update workflow and app status for assembly: %s"
                  % assembly_id)
        LOG.exception(ex)


class Handler(object):
    def __init__(self):
        super(Handler, self).__init__()
        objects.load()

    def echo(self, ctxt, message):
        LOG.debug("%s" % message)

    def _get_stack_name(self, assembly, prefix_len=100):
        assem_name = assembly.name
        # heat stack name has a max allowable length of 255
        return ''.join([assem_name[:min(len(assem_name), prefix_len)], '-',
                        assembly.uuid])

    def _clean_up_artifacts(self, ctxt, t_logger,
                            logs_resource_id, assem):
        try:
            if cfg.CONF.worker.image_storage == 'swift':
                self._delete_app_artifacts_from_swift(ctxt, t_logger,
                                                      assem)
            elif cfg.CONF.worker.image_storage == 'glance':
                self._delete_app_artifacts_from_glance(ctxt, t_logger,
                                                       assem)
            else:
                LOG.debug("image_storage option %s not recognized." %
                          cfg.CONF.worker.image_storage)
        except Exception as e:
            LOG.exception(e)

        # Delete logs
        try:
            log_handler = userlog_handler.UserlogHandler(ctxt)
            log_handler.delete(logs_resource_id)
        except exception.AuthorizationFailure as authexcp:
            t_logger.log(logging.ERROR, str(authexcp))
            LOG.debug(str(authexcp))
            t_logger.upload()

    def _delete_app_artifacts_from_glance(self, ctxt, t_logger, assem):
        if assem.image_id:
            img = objects.registry.Image.get_by_id(ctxt, assem.image_id)
            if img.external_ref:
                try:
                    glance = solum_glanceclient.GlanceClient(ctxt)
                    glance.delete_image_by_id(img.external_ref)
                except swiftexp.ClientException:
                    msg = "Unable to delete DU image from glance."
                    t_logger.log(logging.ERROR, msg)
                    LOG.debug(msg)
                    t_logger.upload()
                    return
            img.destroy(ctxt)

    def _delete_app_artifacts_from_swift(self, ctxt, t_logger, assem):
        if assem.image_id:
            img = objects.registry.Image.get_by_id(ctxt, assem.image_id)
            if img.docker_image_name:
                img_filename = img.docker_image_name.split('-', 1)[1]
                try:
                    swift = solum_swiftclient.SwiftClient(ctxt)
                    swift.delete_object('solum_du', img_filename)
                except swiftexp.ClientException:
                    msg = "Unable to delete DU image from swift."
                    t_logger.log(logging.ERROR, msg)
                    LOG.debug(msg)
                    t_logger.upload()
                    return
            img.destroy(ctxt)

    def destroy_assembly(self, ctxt, assem_id):
        update_assembly(ctxt, assem_id,
                        {'status': STATES.DELETING})

        assem = objects.registry.Assembly.get_by_id(ctxt, assem_id)

        app_obj = get_app_by_assem_id(ctxt, assem.id)
        LOG.debug("Deleting app %s" % app_obj.name)

        logs_resource_id = assem.uuid
        stack_id = self._find_id_if_stack_exists(assem)
        try:
            workflow = objects.registry.Workflow.get_by_assembly_id(assem_id)
            workflow_id = workflow.id
            wf_found = True
        except exception.ResourceNotFound:
            # get_by_assembly_id would result in ResourceNotFound exception
            # if assembly is created directly. In that case use assembly_id
            # with TenantLogger
            workflow_id = assem.uuid
            wf_found = False

        # TODO(devkulkarni) Delete t_logger when returning from this call.
        # This needs to be implemented as a context since there are
        # multiple return paths from this method.
        t_logger = tlog.TenantLogger(ctxt,
                                     assem,
                                     workflow_id,
                                     deployer_log_dir,
                                     'delete')
        msg = "Deleting Assembly %s Workflow %s" % (assem.uuid, workflow_id)
        t_logger.log(logging.DEBUG, msg)
        LOG.debug(msg)

        if stack_id is None:
            t_logger.upload()
            self._clean_up_artifacts(ctxt, t_logger, logs_resource_id, assem)
            if not wf_found:
                assem.destroy(ctxt)
            return
        else:
            # Get the heat client
            heat_clnt = get_heat_client(ctxt, app_obj)
            try:
                t_logger.log(logging.DEBUG, "Deleting Heat stack.")
                LOG.debug("Deleting Heat stack %s", stack_id)
                heat_clnt.stacks.delete(stack_id)
            except exc.HTTPNotFound:
                # stack already deleted
                t_logger.log(logging.ERROR, "Heat stack not found.")
                t_logger.upload()
                self._clean_up_artifacts(ctxt, t_logger, logs_resource_id,
                                         assem)
                if not wf_found:
                    assem.destroy(ctxt)
                return
            except Exception as e:
                LOG.exception(e)
                update_assembly(ctxt, assem_id,
                                {'status': STATES.ERROR_STACK_DELETE_FAILED})
                t_logger.log(logging.ERROR, "Error deleting heat stack.")
                t_logger.upload()
                return

            wait_interval = cfg.CONF.deployer.wait_interval
            growth_factor = cfg.CONF.deployer.growth_factor
            stack_name = self._get_stack_name(assem)
            t_logger.log(logging.DEBUG, "Checking if Heat stack was deleted.")
            for count in range(cfg.CONF.deployer.max_attempts):
                try:
                    # Must use stack_name for expecting a 404
                    heat_clnt.stacks.get(stack_name)
                except exc.HTTPNotFound:
                    t_logger.log(logging.DEBUG, "Stack delete successful.")
                    t_logger.upload()
                    self._clean_up_artifacts(ctxt, t_logger, logs_resource_id,
                                             assem)
                    if not wf_found:
                        assem.destroy(ctxt)
                    return
                time.sleep(wait_interval)
                wait_interval *= growth_factor

            update_assembly(ctxt, assem_id,
                            {'status': STATES.ERROR_STACK_DELETE_FAILED})

            t_logger.log(logging.ERROR, "Error deleting heat stack.")
            t_logger.upload()

    def _destroy_other_assemblies(self, ctxt, assembly_id, heat_clnt):
        # Except current app's stack, destroy all other app stacks

        # We query the newly deployed assembly's object here to
        # ensure that we get most up-to-date value for created_at attribute.
        # If we use the already available object then there is a possibility
        # that the attribute does not have the most up-to-date value due to the
        # possibility that SQLAlchemy might not synchronize object's db state
        # with its in-memory representation.

        new_assembly = objects.registry.Assembly.get_by_id(ctxt, assembly_id)

        # Fetch all assemblies by plan id, and self.destroy() them.
        new_assem_id = new_assembly.id
        app_id = new_assembly.plan_id
        created_at = new_assembly.created_at
        assemblies = objects.registry.AssemblyList.get_earlier(
            new_assem_id,
            app_id,
            STATES.DEPLOYMENT_COMPLETE,
            created_at)
        for assem in assemblies:
            if assem.id == new_assembly.id:
                continue

            # Just delete the old heat stacks and don't delete assemblies
            stack_id = self._find_id_if_stack_exists(assem)
            try:
                LOG.debug("Deleting Heat stack %s", stack_id)
                heat_clnt.stacks.delete(stack_id)
            except exc.HTTPNotFound:
                # stack already deleted
                LOG.debug("Heat stack not found %s", stack_id)
                continue
            except Exception as e:
                LOG.exception(e)
                continue

            # wait for deletion to complete
            wait_interval = cfg.CONF.deployer.wait_interval
            growth_factor = cfg.CONF.deployer.growth_factor
            stack_name = self._get_stack_name(assem)
            for count in range(cfg.CONF.deployer.max_attempts):
                try:
                    # Must use stack_name for expecting a 404
                    heat_clnt.stacks.get(stack_name)
                except exc.HTTPNotFound:
                    LOG.debug("Heat stack deleted %s", stack_id)
                    continue
                time.sleep(wait_interval)
                wait_interval *= growth_factor

    def destroy_app(self, ctxt, app_id):
        # Get all the workflows for app_id
        workflows = objects.registry.WorkflowList.get_all(ctxt, app_id)

        # For each workflow get the assembly and call destroy assembly to
        # remove heat stacks.
        for workflow in workflows:
            assemblie = objects.registry.Assembly.get_by_id(ctxt,
                                                            workflow.assembly)
            self.destroy_assembly(ctxt, assemblie.id)

        # Delete workflow based on app_id
        try:
            objects.registry.Workflow.destroy(app_id)
        except Exception:
            LOG.error("Workflow for app %s not found ", app_id)

        # Delete app
        try:
            db_obj = objects.registry.App.get_by_uuid(ctxt, app_id)
            db_obj.destroy(ctxt)
        except exception.ResourceNotFound:
            LOG.error("App %s not found ", app_id)

    def scale(self, ctxt, assembly_id):
        # TODO(devkulkarni) Find out scale target by querying the app table
        # deploy that many number of dus
        raise exception.NotImplemented()

    def deploy(self, ctxt, assembly_id, image_loc, image_name, ports):

        app_obj = get_app_by_assem_id(ctxt, assembly_id)

        LOG.debug("Deploying app %s" % app_obj.name)

        save_du_ref_for_scaling(ctxt, assembly_id, du=image_loc)

        # Get the heat client
        heat_clnt = get_heat_client(ctxt, app_obj)

        # Get reference to OpenStackClients as it is used to get a reference
        # to neutron client for getting networking parameters
        osc = clients.OpenStackClients(ctxt)

        assem = objects.registry.Assembly.get_by_id(ctxt,
                                                    assembly_id)

        workflow = objects.registry.Workflow.get_by_assembly_id(assembly_id)

        # TODO(devkulkarni) Delete t_logger when returning from this call.
        # This needs to be implemented as a decorator since there are
        # multiple return paths from this method.
        t_logger = tlog.TenantLogger(ctxt,
                                     assem,
                                     workflow.id,
                                     deployer_log_dir,
                                     'deploy')
        msg = "Deploying Assembly %s Workflow %s" % (assem.uuid, workflow.id)
        t_logger.log(logging.DEBUG, msg)

        LOG.debug("Image loc:%s, image_name:%s" % (image_loc, image_name))

        parameters = self._get_parameters(ctxt, cfg.CONF.api.image_format,
                                          image_loc, image_name, assem,
                                          ports, osc, t_logger)
        LOG.debug(parameters)

        if parameters is None:
            return

        template = self._get_template(ctxt, cfg.CONF.api.image_format,
                                      cfg.CONF.worker.image_storage, image_loc,
                                      image_name, assem, ports, t_logger)
        LOG.debug(template)

        if template is None:
            return

        stack_name = self._get_stack_name(assem)
        stack_id = self._find_id_if_stack_exists(assem)

        if assem.status == STATES.DELETING:
            t_logger.log(logging.DEBUG, "Assembly being deleted..returning")
            t_logger.upload()
            return

        if stack_id is not None:
            try:
                heat_clnt.stacks.update(stack_id,
                                        stack_name=stack_name,
                                        template=template,
                                        parameters=parameters)

            except Exception as e:
                LOG.error("Error updating Heat Stack for,"
                          " assembly %s" % assembly_id)
                LOG.exception(e)
                update_assembly(ctxt, assembly_id, {'status': STATES.ERROR})
                t_logger.log(logging.ERROR, "Error updating heat stack.")
                t_logger.upload()
                return
        else:
            try:
                getfile_key = "robust-du-handling.sh"
                file_cnt = None

                try:
                    file_cnt = catalog.get_from_contrib(getfile_key)
                except exception.ObjectNotFound as onf_ex:
                    LOG.exception(onf_ex)
                    update_assembly(ctxt, assem.id, {'status': STATES.ERROR})
                    t_logger.log(logging.ERROR, "Error reading %s"
                                 % getfile_key)
                    t_logger.upload()
                    return

                get_file_dict = {}
                get_file_dict[getfile_key] = file_cnt

                created_stack = heat_clnt.stacks.create(stack_name=stack_name,
                                                        template=template,
                                                        parameters=parameters,
                                                        files=get_file_dict)
            except Exception as exp:
                LOG.error("Error creating Heat Stack for,"
                          " assembly %s" % assembly_id)
                LOG.exception(exp)
                update_assembly(ctxt, assembly_id,
                                {'status': STATES.ERROR_STACK_CREATE_FAILED})
                t_logger.log(logging.ERROR, "Error creating heat stack.")
                t_logger.upload()
                return
            stack_id = created_stack['stack']['id']

            LOG.debug("Stack id: %s" % stack_id)

            comp_name = 'Heat_Stack_for_%s' % assem.name
            comp_description = 'Heat Stack %s' % (
                yaml.safe_load(template).get('description'))
            try:
                objects.registry.Component.assign_and_create(
                    ctxt, assem, comp_name, 'heat_stack', comp_description,
                    created_stack['stack']['links'][0]['href'], stack_id)
            except sqla_exc.IntegrityError:
                LOG.error("IntegrityError in creating Heat Stack component,"
                          " assembly %s may be deleted" % assembly_id)
                update_assembly(ctxt, assembly_id, {'status': STATES.ERROR})
                t_logger.log(logging.ERROR, "Error creating heat stack.")
                t_logger.upload()
                return
        update_assembly(ctxt, assembly_id, {'status': STATES.DEPLOYING})

        result = self._check_stack_status(ctxt, assembly_id, heat_clnt,
                                          stack_id, ports, t_logger)
        assem.status = result
        t_logger.upload()
        if result == STATES.DEPLOYMENT_COMPLETE:
            self._destroy_other_assemblies(ctxt, assembly_id, heat_clnt)

    def _get_template(self, ctxt, image_format, image_storage,
                      image_loc, image_name, assem, ports, t_logger):
        template = None

        if image_format == 'docker':
            try:
                template = catalog.get('templates', 'basic')
            except exception.ObjectNotFound as onf_ex:
                LOG.exception(onf_ex)
                update_assembly(ctxt, assem.id, {'status': STATES.ERROR})
                t_logger.log(logging.ERROR, "Error reading heat template.")
                t_logger.upload()
                return template

        elif image_format == 'vm':
            if image_storage == 'glance':
                msg = ("image_storage %s not supported with image_format %s" %
                       (image_storage, image_format))
                LOG.debug(msg)
                update_assembly(ctxt, assem.id, {'status': STATES.ERROR})
                t_logger.log(logging.DEBUG, "Solum config error: %s " % msg)
                t_logger.upload()
            else:
                try:
                    template = catalog.get('templates', 'coreos')
                except exception.ObjectNotFound as onf_ex:
                    LOG.exception(onf_ex)
                    update_assembly(ctxt, assem.id, {'status': STATES.ERROR})
                    t_logger.log(logging.ERROR, "Error reading heat template.")
                    t_logger.upload()
                    return template

                if image_storage == 'docker_registry':
                    template = self._get_template_for_docker_reg(
                        assem, template, image_loc, image_name, ports)
        else:
            LOG.debug("Image format %s is not supported." % image_format)
            update_assembly(ctxt, assem.id, {'status': STATES.ERROR})
            t_logger.log(logging.DEBUG, "Solum config error: Image format.")
            t_logger.upload()

        return template

    def _get_parameters(self, ctxt, image_format, image_loc, image_name,
                        assem, ports, osc, t_logger):
        parameters = None
        if image_format == 'docker':
            glance_img_uuid = image_loc
            LOG.debug("Image id:%s" % glance_img_uuid)
            LOG.debug("Specified ports:%s" % ports)
            LOG.debug("Picking first port..")
            port_to_use = ports[0]
            LOG.debug("Application port:%s" % port_to_use)

            parameters = {'app_name': assem.name,
                          'image': glance_img_uuid,
                          'port': port_to_use}
            parameters.update(heat_utils.get_network_parameters(osc))

        elif image_format == 'vm':
            parameters = {'name': str(assem.uuid),
                          'flavor': cfg.CONF.deployer.flavor,
                          'key_name': cfg.CONF.deployer.key_name,
                          'image': cfg.CONF.deployer.image}
            ports_str = ''
            for port in ports:
                ports_str += ' -p {pt}:{pt}'.format(pt=port)

            parameters['location'] = image_loc
            parameters['du'] = image_name
            parameters['publish_ports'] = ports_str.strip()
        else:
            LOG.debug("Image format %s is not supported." % image_format)
            update_assembly(ctxt, assem.id, {'status': STATES.ERROR})
            t_logger.log(logging.DEBUG, "Solum config error: Image format.")
            t_logger.upload()
        return parameters

    def _check_stack_status(self, ctxt, assembly_id, heat_clnt, stack_id,
                            ports, t_logger):

        wait_interval = cfg.CONF.deployer.wait_interval
        growth_factor = cfg.CONF.deployer.growth_factor

        stack = None

        for count in range(cfg.CONF.deployer.max_attempts):
            time.sleep(wait_interval)
            wait_interval *= growth_factor
            try:
                stack = heat_clnt.stacks.get(stack_id)
            except Exception as e:
                LOG.exception(e)
                continue

            if stack.status == 'COMPLETE':
                break
            elif stack.status == 'FAILED':
                update_assembly(ctxt, assembly_id,
                                {'status': STATES.ERROR_STACK_CREATE_FAILED})
                lg_msg = "App deployment failed: Heat stack creation failure"
                t_logger.log(logging.ERROR, lg_msg)
                return STATES.ERROR_STACK_CREATE_FAILED

        if stack is None or (stack and stack.status == ""):
            update_assembly(ctxt, assembly_id,
                            {'status': STATES.ERROR_STACK_CREATE_FAILED})
            lg_msg = "App deployment failed: Heat stack is in unexpected state"
            t_logger.log(logging.ERROR, lg_msg)
            return STATES.ERROR_STACK_CREATE_FAILED

        host_ip = self._parse_server_ip(stack)
        if host_ip is None:
            LOG.exception("Could not parse url from heat stack.")
            update_assembly(ctxt, assembly_id,
                            {'status': STATES.ERROR})
            lg_msg = ("App deployment failed: "
                      "container IP address not available")
            t_logger.log(logging.ERROR, lg_msg)
            return STATES.ERROR

        app_uri = host_ip

        if len(ports) == 1:
            app_uri += ":" + str(ports[0])
        if len(ports) > 1:
            port_list = ','.join(str(p) for p in ports)
            app_uri += ":[" + port_list + "]"

        to_upd = {'status': STATES.STARTING_APP,
                  'application_uri': app_uri}
        update_assembly(ctxt, assembly_id, to_upd)
        LOG.debug("Application URI: %s" % app_uri)

        successful_ports = set()
        du_is_up = False
        for count in range(cfg.CONF.deployer.du_attempts):
            for prt in ports:
                if prt not in successful_ports:
                    du_url = 'http://{host}:{port}'.format(host=host_ip,
                                                           port=prt)
                    try:
                        if repo_utils.is_reachable(du_url):
                            successful_ports.add(prt)
                            if len(successful_ports) == len(ports):
                                du_is_up = True
                                break
                    except socket.timeout:
                        LOG.debug("Connection to %s timed out"
                                  "assembly ID: %s" % (du_url, assembly_id))
                    except (httplib2.HttpLib2Error, socket.error) as serr:
                        if count % 5 == 0:
                            LOG.exception(serr)
                        else:
                            LOG.debug(".")
                    except Exception as exp:
                        LOG.exception(exp)
                        update_assembly(ctxt, assembly_id,
                                        {'status': STATES.ERROR})
                        lg_msg = ("App deployment error: unexpected error "
                                  " when trying to reach app endpoint")
                        t_logger.log(logging.ERROR, lg_msg)
                        return STATES.ERROR
            if du_is_up:
                break
            time.sleep(1)

        if du_is_up:
            to_update = {'status': STATES.DEPLOYMENT_COMPLETE}
        else:
            to_update = {'status': STATES.ERROR_CODE_DEPLOYMENT}
            lg_msg = ("App deployment error: unreachable server or port, "
                      " please check your port config.")
            t_logger.log(logging.ERROR, lg_msg)
        update_assembly(ctxt, assembly_id, to_update)
        return to_update['status']

    def _parse_server_ip(self, heat_output):
        """Parse server ip from heat-stack-show output."""
        if 'outputs' in heat_output._info:
            for outputs in heat_output._info['outputs']:
                if outputs['output_key'] == 'public_ip':
                    return outputs['output_value']
        return None

    def _find_id_if_stack_exists(self, assem):
        if assem.heat_stack_component is not None:
            return assem.heat_stack_component.heat_stack_id
        return None

    def _get_template_for_docker_reg(self, assem, template,
                                     image_loc, image_name, ports):
        du_name = image_loc
        ports_str = ''
        for port in ports:
            ports_str += ' -p {pt}:{pt}'.format(pt=port)
        run_docker_str = ('#!/bin/bash -x\n'
                          '# Invoke the container\n'
                          'docker run {publish_ports} -d {du}\n'
                          'wc_notify --data-binary {stat}')
        run_docker = run_docker_str.format(publish_ports=ports_str.strip(),
                                           du=du_name,
                                           stat='\'{"status": "SUCCESS"}\'')

        LOG.debug("run_docker:%s" % run_docker)

        template_bdy = yaml.safe_load(template)
        comp_instance = template_bdy['resources']['compute_instance']
        user_data = comp_instance['properties']['user_data']
        user_data['str_replace']['template'] = run_docker
        comp_instance['properties']['user_data'] = user_data
        template_bdy['resources']['compute_instance'] = comp_instance
        template = yaml.dump(template_bdy)
        return template



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.api.controllers.camp.v1_1 import formats
from solum import objects
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
class TestFormats(base.BaseTestCase):
    def setUp(self):
        super(TestFormats, self).setUp()
        objects.load()

    # These tests aren't strictly "unit tests" since we don't stub-out the
    # handler for format resources. However, since that handler simply
    # looks up a static object in a static dictionary, it isn't that big
    # of a deal.

    def test_formats_get(self, resp_mock, request_mock):
        cont = formats.FormatsController()
        resp = cont.get()
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertEqual('formats', resp['result'].type)
        self.assertEqual('Solum_CAMP_formats', resp['result'].name)
        format_links = resp['result'].format_links
        self.assertEqual(1, len(format_links))
        self.assertEqual('JSON', format_links[0].target_name)

    def test_get_json_format(self, resp_mock, request_mock):
        cont = formats.FormatsController()
        resp = cont.get_one('json_format')
        self.assertIsNotNone(resp)
        self.assertEqual(200, resp_mock.status)
        self.assertEqual('format', resp['result'].type)
        self.assertEqual('JSON', resp['result'].name)

    def test_format_get_not_found(self, resp_mock, request_mock):
        cont = formats.FormatsController()
        resp = cont.get_one('nonexistent_format')
        self.assertIsNotNone(resp)
        self.assertEqual(404, resp_mock.status)



# Copyright 2015 - Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sqlalchemy as sa
from sqlalchemy.orm import exc as orm_exc

from solum.objects import app as abstract
from solum.objects.sqlalchemy import models as sql


class App(sql.Base, abstract.App):
    """Represent an app in sqlalchemy."""

    __tablename__ = 'app'
    __resource__ = 'apps'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.String(36), primary_key=True)
    deleted = sa.Column(sa.Boolean, default=False)
    project_id = sa.Column(sa.String(36))
    user_id = sa.Column(sa.String(36))

    languagepack = sa.Column(sa.String(1024))
    stack_id = sa.Column(sa.String(36))

    name = sa.Column(sa.String(length=100))
    description = sa.Column(sa.String(length=255))
    ports = sa.Column(sql.YAMLEncodedDict(1024))
    source = sa.Column(sql.YAMLEncodedDict(1024))
    workflow_config = sa.Column(sql.YAMLEncodedDict(1024))
    trigger_uuid = sa.Column(sa.String(length=36))
    trigger_actions = sa.Column(sql.YAMLEncodedDict(1024))
    trust_id = sa.Column(sa.String(255))
    trust_user = sa.Column(sa.String(256))
    status = sa.Column(sa.String(36))
    app_url = sa.Column(sa.String(1024))
    raw_content = sa.Column(sa.String(2048))
    scale_config = sa.Column(sql.JSONEncodedDict(1024))

    def _non_updatable_fields(self):
        return set(('id', 'project_id'))

    @classmethod
    def get_by_uuid(cls, context, item_uuid):
        app = cls.get_by_id(context, item_uuid)
        return app

    @classmethod
    def get_by_trigger_id(cls, context, trigger_id):
        try:
            session = sql.Base.get_session()
            return session.query(cls).filter_by(trigger_uuid=trigger_id).one()
        except sa.orm.exc.NoResultFound:
            cls._raise_trigger_not_found(trigger_id)

    @classmethod
    def get_all_by_lp(cls, context, lp):
        session = sql.SolumBase.get_session()
        apps = []
        with session.begin():
            query = session.query(cls).filter_by(languagepack=lp)
            apps = sql.filter_by_project(context, query).all()
            return apps

    @classmethod
    @sql.retry
    def update_and_save(cls, context, id_or_uuid, data):
        try:
            session = sql.SolumBase.get_session()
            with session.begin():
                query = session.query(cls).filter_by(id=id_or_uuid)
                obj = sql.filter_by_project(context, query).one()
                if obj._is_updatable():
                    obj.update(data)
                    session.merge(obj)
            return obj
        except orm_exc.NoResultFound:
            cls._raise_not_found(id_or_uuid)


class AppList(abstract.AppList):
    """Represent a list of apps in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return AppList(sql.model_query(context, App))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock

from oslo_db import exception as db_exc
import pecan
import yaml

from solum.api.controllers.v1.datamodel import plan as planmodel
from solum.api.controllers.v1 import plan
from solum.common import exception
from solum import objects
from solum.tests import base
from solum.tests import fakes


class TestPlanModuleFunctions(base.BaseTestCase):

    @mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
    def test_yaml_content(self, mock_req):
        m = fakes.FakePlan()
        ref_content = plan.yaml_content(m)
        host_url = pecan.request.application_url.rstrip('/')
        self.assertEqual(ref_content['uri'], '%s/v1/plans/%s' %
                                             (host_url, m.uuid))

    @mock.patch('solum.api.controllers.v1.plan.init_plan_v1')
    def test_init_plan_by_version(self, init_plan_v1):
        yml_input_plan = {'version': 1, 'name': 'plan1', 'description': 'dsc'}
        plan.init_plan_by_version(yml_input_plan)
        self.assertTrue(init_plan_v1.called)

    @mock.patch('solum.api.controllers.v1.plan.init_plan_v1')
    def test_init_plan_by_version_missing(self, init_plan_v1):
        yml_input_plan = {'name': 'plan1', 'description': 'dsc'}
        self.assertRaises(exception.BadRequest, plan.init_plan_by_version,
                          yml_input_plan)

    @mock.patch('solum.api.controllers.v1.plan.init_plan_v1')
    def test_init_plan_by_version_not_existing(self, init_plan_v1):
        yml_input_plan = {'version': 424242424242424242, 'name': 'plan1',
                          'description': 'dsc'}
        self.assertRaises(exception.BadRequest, plan.init_plan_by_version,
                          yml_input_plan)

    @mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
    def test_init_plan_v1(self, mock_req):
        yml_input_plan = {'version': 1, 'name': 'plan1', 'description': 'dsc'}
        plan_v1 = plan.init_plan_v1(yml_input_plan)
        self.assertIsInstance(plan_v1, planmodel.Plan)

    @mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
    def test_init_plan_v1_bad_names(self, mock_req):
        yml_input_plan = {'version': 1, 'name': 'a' * 101}
        self.assertRaises(exception.BadRequest, plan.init_plan_v1,
                          yml_input_plan)

        yml_input_plan = {'version': 1, 'name': 'name with spaces'}
        self.assertRaises(exception.BadRequest, plan.init_plan_v1,
                          yml_input_plan)


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.plan_handler.PlanHandler')
class TestPlanController(base.BaseTestCase):
    def setUp(self):
        super(TestPlanController, self).setUp()
        objects.load()

    def test_plan_get_yaml(self, PlanHandler, resp_mock,
                           request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.accept = 'application/x-yaml'
        hand_get = PlanHandler.return_value.get
        fake_plan = fakes.FakePlan()
        hand_get.return_value = fake_plan
        cont = plan.PlanController('test_id')
        resp = cont.get()
        self.assertIsNotNone(resp)
        resp_yml = yaml.safe_load(resp)
        self.assertEqual(fake_plan.raw_content['name'], resp_yml['name'])
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_plan_get_json(self, PlanHandler, resp_mock,
                           request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PlanHandler.return_value.get
        fake_plan = fakes.FakePlan()
        hand_get.return_value = fake_plan
        cont = plan.PlanController('test_id')
        resp = cont.get()
        self.assertIsNotNone(resp)
        resp_json = json.loads(resp)
        self.assertEqual(fake_plan.name, resp_json['name'])
        self.assertEqual(fake_plan.project_id,
                         resp_json['project_id'])
        self.assertEqual(fake_plan.uuid, resp_json['uuid'])
        hand_get.assert_called_with('test_id')
        self.assertEqual(200, resp_mock.status)

    def test_plan_get_not_found_yaml(self, PlanHandler,
                                     resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.accept = 'application/x-yaml'
        hand_get = PlanHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(name='plan',
                                                          id='test_id')
        plan.PlanController('test_id').get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_plan_get_not_found_json(self, PlanHandler,
                                     resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PlanHandler.return_value.get
        hand_get.side_effect = exception.ResourceNotFound(name='plan',
                                                          id='test_id')
        plan.PlanController('test_id').get()
        hand_get.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_plan_put_none(self, PlanHandler, resp_mock,
                           request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.content_type = 'application/x-yaml'
        request_mock.body = ''
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_needs_name(self, PlanHandler, resp_mock,
                                 request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.content_type = 'application/x-yaml'
        request_mock.body = 'version: 1\n'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_bad_names(self, PlanHandler, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.content_type = 'application/x-yaml'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        pc = plan.PlanController('test_id')
        request_mock.body = 'version: 1\nname: %s' % ('a' * 101)
        pc.put()
        self.assertEqual(400, resp_mock.status)

        request_mock.body = 'version: 1\nname: %s' % ('name with spaces')
        pc.put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_invalid_yaml(self, PlanHandler, resp_mock,
                                   request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.content_type = 'application/x-yaml'
        request_mock.body = 'invalid yaml file'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_empty_yaml(self, PlanHandler, resp_mock,
                                 request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.content_type = 'application/x-yaml'
        request_mock.body = '{}'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_not_found(self, PlanHandler, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        data = 'version: 1\nname: ex_plan1\ndescription: dsc1.'
        request_mock.body = data
        request_mock.content_type = 'application/x-yaml'
        hand_update = PlanHandler.return_value.update
        hand_update.side_effect = exception.ResourceNotFound(
            name='plan', id='test_id')
        plan.PlanController('test_id').put()
        hand_update.assert_called_with('test_id', {'name': 'ex_plan1',
                                                   'description': u'dsc1.'})
        self.assertEqual(404, resp_mock.status)

    def test_plan_put_ok_yaml(self, PlanHandler, resp_mock,
                              request_mock, mock_policy):
        mock_policy.return_value = True
        data = 'version: 1\nname: ex_plan1\ndescription: dsc1.'
        request_mock.body = data
        request_mock.content_type = 'application/x-yaml'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        hand_update.assert_called_with('test_id', {'name': 'ex_plan1',
                                                   'description': u'dsc1.'})
        self.assertEqual(200, resp_mock.status)

    def test_plan_put_ok_json(self, PlanHandler, resp_mock,
                              request_mock, mock_policy):
        mock_policy.return_value = True
        data = {'name': 'foo', 'version': '1'}
        request_mock.body = json.dumps(data)
        request_mock.content_type = 'application/json'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        hand_update.assert_called_with('test_id', {'name': 'foo'})
        self.assertEqual(200, resp_mock.status)

    def test_plan_put_wrong_version_yaml(self, PlanHandler,
                                         resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        data = 'name: ex_plan1\ndescription: yaml plan1.\nversion: 42'
        request_mock.body = data
        request_mock.content_type = 'application/x-yaml'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_version_not_found_yaml(self, PlanHandler,
                                             resp_mock, request_mock,
                                             mock_policy):
        mock_policy.return_value = True
        data = 'name: ex_plan1\ndescription: yaml plan1.'
        request_mock.body = data
        request_mock.content_type = 'application/x-yaml'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_wrong_version_json(self, PlanHandler,
                                         resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        data = {'name': 'foo', 'version': '42'}
        request_mock.body = json.dumps(data)
        request_mock.content_type = 'application/json'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_put_version_not_found_json(self, PlanHandler,
                                             resp_mock, request_mock,
                                             mock_policy):
        mock_policy.return_value = True
        data = {'name': 'foo'}
        request_mock.body = json.dumps(data)
        request_mock.content_type = 'application/json'
        hand_update = PlanHandler.return_value.update
        hand_update.return_value = fakes.FakePlan()
        plan.PlanController('test_id').put()
        self.assertEqual(400, resp_mock.status)

    def test_plan_delete_not_found(self, PlanHandler, resp_mock,
                                   request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = PlanHandler.return_value.delete
        hand_delete.side_effect = exception.ResourceNotFound(
            name='plan', plan_id='test_id')
        obj = plan.PlanController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(404, resp_mock.status)

    def test_plan_delete_ok(self, PlanHandler, resp_mock,
                            request_mock, mock_policy):
        mock_policy.return_value = True
        hand_delete = PlanHandler.return_value.delete
        hand_delete.return_value = None
        obj = plan.PlanController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(202, resp_mock.status)

    def test_plan_delete_dberror(self, PlanHandler, resp_mock,
                                 request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PlanHandler.return_value.get
        fake_plan = fakes.FakePlan()
        hand_get.return_value = fake_plan

        hand_delete = PlanHandler.return_value.delete
        hand_delete.side_effect = db_exc.DBError()
        obj = plan.PlanController('test_id')
        obj.delete()
        hand_delete.assert_called_with('test_id')
        self.assertEqual(409, resp_mock.status)


@mock.patch('solum.common.policy.check')
@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.handlers.plan_handler.PlanHandler')
class TestPlansController(base.BaseTestCase):
    def setUp(self):
        super(TestPlansController, self).setUp()
        objects.load()

    def test_plans_get_all_yaml(self, PlanHandler, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PlanHandler.return_value.get_all
        request_mock.accept = 'application/x-yaml'
        fake_plan = fakes.FakePlan()
        hand_get.return_value = [fake_plan]
        resp = plan.PlansController().get_all()
        self.assertIsNotNone(resp)
        resp_yml = yaml.safe_load(resp)
        self.assertEqual(fake_plan.raw_content['name'], resp_yml[0]['name'])
        self.assertEqual(200, resp_mock.status)
        hand_get.assert_called_with()

    def test_plans_get_all_json(self, PlanHandler, resp_mock,
                                request_mock, mock_policy):
        mock_policy.return_value = True
        hand_get = PlanHandler.return_value.get_all
        fake_plan = fakes.FakePlan()
        hand_get.return_value = [fake_plan]
        resp = plan.PlansController().get_all()
        self.assertIsNotNone(resp)
        resp_json = json.loads(resp)
        self.assertEqual(fake_plan.raw_content['name'], resp_json[0]['name'])
        self.assertEqual(200, resp_mock.status)
        hand_get.assert_called_with()

    def test_plans_post_yaml(self, PlanHandler, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = 'version: 1\nname: ex_plan1\ndescription: dsc1.'
        request_mock.content_type = 'application/x-yaml'
        hand_create = PlanHandler.return_value.create
        hand_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        hand_create.assert_called_with({'name': 'ex_plan1',
                                        'description': 'dsc1.'})
        self.assertEqual(201, resp_mock.status)

    def test_plans_post_json(self, PlanHandler, resp_mock,
                             request_mock, mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo', 'version': '1'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_create = PlanHandler.return_value.create
        hand_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        hand_create.assert_called_with({'name': 'foo'})
        self.assertEqual(201, resp_mock.status)

    def test_plans_post_wrong_version_yaml(self, PlanHandler,
                                           resp_mock, request_mock,
                                           mock_policy):
        mock_policy.return_value = True
        request_mock.body = 'version: 42\nname: ex_plan1\ndescription: dsc1.'
        request_mock.content_type = 'application/x-yaml'
        hand_create = PlanHandler.return_value.create
        hand_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        self.assertEqual(400, resp_mock.status)

    def test_plans_post_version_not_found_yaml(self, PlanHandler,
                                               resp_mock, request_mock,
                                               mock_policy):
        mock_policy.return_value = True
        request_mock.body = 'name: ex_plan1\ndescription: dsc1.'
        request_mock.content_type = 'application/x-yaml'
        hand_create = PlanHandler.return_value.create
        hand_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        self.assertEqual(400, resp_mock.status)

    def test_plans_post_wrong_version_json(self, PlanHandler,
                                           resp_mock, request_mock,
                                           mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo', 'version': '42'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_create = PlanHandler.return_value.create
        hand_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        self.assertEqual(400, resp_mock.status)

    def test_plans_post_version_not_found_json(self, PlanHandler,
                                               resp_mock, request_mock,
                                               mock_policy):
        mock_policy.return_value = True
        json_update = {'name': 'foo'}
        request_mock.body = json.dumps(json_update)
        request_mock.content_type = 'application/json'
        hand_create = PlanHandler.return_value.create
        hand_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        self.assertEqual(400, resp_mock.status)

    def test_plans_post_nodata(self, handler_mock, resp_mock,
                               request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = ''
        request_mock.content_type = 'application/json'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        self.assertEqual(400, resp_mock.status)

    def test_plans_post_invalid_yaml(self, handler_mock,
                                     resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = 'invalid yaml file'
        request_mock.content_type = 'application/x-yaml'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        self.assertEqual(400, resp_mock.status)

    def test_plans_post_empty_yaml(self, handler_mock,
                                   resp_mock, request_mock, mock_policy):
        mock_policy.return_value = True
        request_mock.body = '{}'
        request_mock.content_type = 'application/x-yaml'
        handler_create = handler_mock.return_value.create
        handler_create.return_value = fakes.FakePlan()
        plan.PlansController().post()
        self.assertEqual(400, resp_mock.status)


class TestPlanAsDict(base.BaseTestCase):

    scenarios = [
        ('none', dict(data=None)),
        ('one', dict(data={'name': 'foo'})),
        ('full', dict(data={'uri': 'http://example.com/v1/plans/x1',
                            'name': 'Example-plan',
                            'type': 'plan'}))
    ]

    def test_as_dict(self):
        objects.load()
        if self.data is None:
            s = planmodel.Plan()
            self.data = {}
        else:
            s = planmodel.Plan(**self.data)
        if 'uri' in self.data:
            del self.data['uri']
        if 'type' in self.data:
            del self.data['type']

        self.assertEqual(self.data, s.as_dict(objects.registry.Plan))



# Copyright 2014 - Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging
import sqlalchemy as sa

from solum.common import exception
from solum.objects import plan as abstract
from solum.objects.sqlalchemy import models as sql


LOG = logging.getLogger(__name__)


class Plan(sql.Base, abstract.Plan):
    """Represent a plan in sqlalchemy."""

    __resource__ = 'plans'
    __tablename__ = 'plan'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    uuid = sa.Column(sa.String(36))
    project_id = sa.Column(sa.String(36))
    user_id = sa.Column(sa.String(36))
    name = sa.Column(sa.String(255), nullable=False)
    description = sa.Column(sa.String(255))
    raw_content = sa.Column(sql.YAMLEncodedDict(2048))
    trigger_id = sa.Column(sa.String(36))
    trust_id = sa.Column(sa.String(255))
    username = sa.Column(sa.String(256))

    @classmethod
    def _raise_trigger_not_found(cls, item_id):
        """Raise a NotFound exception."""
        raise exception.ResourceNotFound(id=item_id, name='trigger')

    @classmethod
    def get_by_trigger_id(cls, context, trigger_id):
        try:
            session = sql.Base.get_session()
            return session.query(cls).filter_by(trigger_id=trigger_id).one()
        except sa.orm.exc.NoResultFound:
            cls._raise_trigger_not_found(trigger_id)

    @classmethod
    def get_by_uuid(cls, context, app_id):
        try:
            session = sql.Base.get_session()
            return session.query(cls).filter_by(uuid=app_id).one()
        except sa.orm.exc.NoResultFound as e:
            LOG.exception(e)
            raise exception.ResourceNotFound(id=app_id, name='plan')

    def _non_updatable_fields(self):
        return set(('uuid', 'id', 'project_id'))

    def refined_content(self):
        if self.raw_content and self.uuid:
            self.raw_content['uuid'] = self.uuid
        return self.raw_content


class PlanList(abstract.PlanList):
    """Represent a list of plans in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return PlanList(sql.model_query(context, Plan))



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import itertools

from solum.common.policies import assembly
from solum.common.policies import base
from solum.common.policies import component
from solum.common.policies import extension
from solum.common.policies import languagepack
from solum.common.policies import operation
from solum.common.policies import pipeline
from solum.common.policies import plan
from solum.common.policies import sensor
from solum.common.policies import service
from solum.common.policies import trigger


def list_rules():
    return itertools.chain(
        assembly.list_rules(),
        base.list_rules(),
        component.list_rules(),
        extension.list_rules(),
        languagepack.list_rules(),
        operation.list_rules(),
        pipeline.list_rules(),
        plan.list_rules(),
        sensor.list_rules(),
        service.list_rules(),
        trigger.list_rules(),
    )



# Copyright 2015 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import datetime

import wsme
from wsme import types as wtypes

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Workflow(wtypes.Base):
    """Representation of a Workflow.

    A workflow maintains a living creation and deployment of an App.
    """

    # (devkulkarni) Added base_url to get around strict validation
    # checking of WSME 0.8.0
    # https://bugs.launchpad.net/solum/+bug/1491504
    # https://bugs.launchpad.net/solum/+bug/1491499
    base_url = common_types.Uri
    "URI of the base resource."

    uri = common_types.Uri
    "URI to the resource."

    uuid = wtypes.text
    "Unique Identifier of the resource"

    type = wtypes.text
    "The resource type."

    id = wtypes.text
    updated_at = datetime.datetime
    created_at = datetime.datetime
    app_id = wtypes.text
    wf_id = int
    source = wtypes.DictType(wtypes.text, api_types.MultiType(
        wtypes.text,
        int,
        bool,
        float))
    config = {wtypes.text: wtypes.text}
    actions = [wtypes.text]
    du_id = wtypes.text
    status = wtypes.text
    result = wtypes.text
    scale_target = int

    def __init__(self, *args, **kwargs):
        super(Workflow, self).__init__(*args, **kwargs)

    @classmethod
    def sample(cls):
        return cls(
            wf_id=1,
            config={},
            actions={},
            source={},
            status=''
            )

    @classmethod
    def from_db_model(cls, m, host_url):
        json = m.as_dict()
        json['type'] = m.__tablename__
        json['uri'] = ''
        json['uri'] = ('%s/v1/apps/%s/workflows/%s' %
                       (host_url, m.app_id, m.wf_id))
        return cls(**(json))

    def as_dict_from_keys(self, keys):
        return dict((k, getattr(self, k))
                    for k in keys
                    if hasattr(self, k) and
                    getattr(self, k) != wsme.Unset)

    def as_dict(self, db_model):
        valid_keys = [attr for attr in db_model.__dict__.keys()
                      if attr[:2] != '__' and attr != 'as_dict']
        base = self.as_dict_from_keys(valid_keys)
        attrs = [
            'id',
            'app_id',
            'wf_id',
            'source',
            'config',
            'actions',
            'status',
            'result',
            'scale_target'
            ]
        for a in attrs:
            if getattr(self, a) is wsme.Unset:
                continue
            if getattr(self, a) is None:
                continue
            base[a] = getattr(self, a)
        return base



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging
import sqlalchemy

from solum.common import exception
from solum import objects
from solum.objects import pipeline as abstract
from solum.objects.sqlalchemy import execution
from solum.objects.sqlalchemy import models as sql

LOG = logging.getLogger(__name__)


class Pipeline(sql.Base, abstract.Pipeline):
    """Represent an pipeline in sqlalchemy."""

    __resource__ = 'pipelines'
    __tablename__ = 'pipeline'
    __table_args__ = sql.table_args()

    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True,
                           autoincrement=True)
    uuid = sqlalchemy.Column(sqlalchemy.String(36))
    project_id = sqlalchemy.Column(sqlalchemy.String(36))
    user_id = sqlalchemy.Column(sqlalchemy.String(36))
    name = sqlalchemy.Column(sqlalchemy.String(100))
    description = sqlalchemy.Column(sqlalchemy.String(255))
    tags = sqlalchemy.Column(sqlalchemy.Text)
    workbook_name = sqlalchemy.Column(sqlalchemy.String(255))
    plan_id = sqlalchemy.Column(sqlalchemy.Integer,
                                sqlalchemy.ForeignKey('plan.id'),
                                nullable=False)
    trigger_id = sqlalchemy.Column(sqlalchemy.String(36))
    trust_id = sqlalchemy.Column(sqlalchemy.String(255))

    @property
    def plan_uuid(self):
        return objects.registry.Plan.get_by_id(None, self.plan_id).uuid

    @plan_uuid.setter
    def plan_uuid(self, value):
        plan = objects.registry.Plan.get_by_uuid(None, value)
        self.plan_id = plan.id

    @property
    def _extra_keys(self):
        return ['plan_uuid']

    @classmethod
    def get_by_trigger_id(cls, context, trigger_id):
        try:
            session = sql.Base.get_session()
            return session.query(cls).filter_by(trigger_id=trigger_id).one()
        except sqlalchemy.orm.exc.NoResultFound:
            raise exception.ResourceNotFound(id=trigger_id, name='trigger')

    def last_execution(self):
        session = sql.Base.get_session()
        try:
            return session.query(execution.Execution).filter_by(
                pipeline_id=self.id).order_by(
                    sqlalchemy.desc(
                        execution.Execution.created_at)).limit(1).one()
        except sqlalchemy.orm.exc.NoResultFound as no_ex:
            LOG.debug('No Execution found %s' % no_ex)
            return None

    @property
    def executions(self):
        session = sql.Base.get_session()
        return session.query(execution.Execution).filter_by(
            pipeline_id=self.id).all()

    def destroy(self, context):
        session = sql.Base.get_session()
        with session.begin():
            session.query(execution.Execution).filter_by(
                pipeline_id=self.id).delete()
            session.query(self.__class__).filter_by(
                id=self.id).delete()


class PipelineList(abstract.PipelineList):
    """Represent a list of pipelines in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        return PipelineList(sql.model_query(context, Pipeline))



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import execution
from solum.api.handlers import pipeline_handler
from solum.common import exception
from solum.common import policy


class ExecutionsController(rest.RestController):
    """Manages operations on the executions collection."""

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([execution.Execution], wtypes.text)
    def get_all(self, pipeline_id):
        """Return all executions, based on the provided pipeline_id."""
        policy.check('get_pipeline_executions',
                     pecan.request.security_context)
        handler = pipeline_handler.PipelineHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return [execution.Execution.from_db_model(obj, host_url)
                for obj in handler.get(pipeline_id).executions]



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from solum.common import exception
from solum.objects import registry
from solum.objects.sqlalchemy import assembly
from solum.objects.sqlalchemy import component
from solum.tests import base
from solum.tests import utils


class TestComponent(base.BaseTestCase):
    def setUp(self):
        super(TestComponent, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()
        self.data_assembly = [
            {'project_id': self.ctx.project_id,
             'uuid': 'ce43e347f0b0422825245b3e5f140a81cef6e65b',
             'user_id': 'fred',
             'name': 'assembly1',
             'description': 'test assembly',
             'trigger_id': 'trigger-uuid-1234',
             'tags': 'assembly tags',
             'plan_id': 'plan_id_1',
             'status': 'Building',
             'application_uri': 'http://192.168.78.21:5000'}]
        utils.create_models_from_data(assembly.Assembly, self.data_assembly,
                                      self.ctx)

        self.data = [{'project_id': self.ctx.project_id,
                      'user_id': 'fred',
                      'uuid': 'ce43e347f0b0422825245b3e5f140a81cef6e65b',
                      'name': 'component_no_assembly',
                      'component_type': 'xyz',
                      'description': 'test component',
                      'parent_component_id': '87d98s',
                      'tags': 'component tags',
                      'heat_stack_id': '4c712026-dcd5-4664-90b8-0915494c1332'},
                     {'project_id': self.ctx.project_id,
                      'user_id': 'fred',
                      'uuid': '70763488-72e0-44ac-a612-e94bf5488555',
                      'name': 'component_assembly',
                      'component_type': 'xyz',
                      'description': 'test component',
                      'assembly_uuid': 'ce43e347f0b042282524'
                                       '5b3e5f140a81cef6e65b',
                      'parent_component_id': '87d98s',
                      'tags': 'component tags',
                      'heat_stack_id': '4c712026-dcd5-4664-90b8-0915494c1332'}]
        utils.create_models_from_data(component.Component, self.data, self.ctx)

    def test_objects_registered(self):
        self.assertTrue(registry.Component)
        self.assertTrue(registry.ComponentList)

    def test_get_all(self):
        lst = component.ComponentList()
        self.assertEqual(2, len(lst.get_all(self.ctx)))

    def test_check_data(self):
        ta = component.Component().get_by_id(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(ta, key))

    def test_assembly_extra_key(self):
        ta = component.Component().get_by_id(self.ctx, self.data[1]['id'])
        self.assertEqual(ta.assembly_uuid, self.data_assembly[0]['uuid'])

    def test_assembly_extra_key_optional(self):
        ta = component.Component().get_by_id(self.ctx, self.data[0]['id'])
        self.assertIsNone(ta.assembly_uuid)

    def test_assembly_extra_key_not_found(self):
        self.assertRaises(exception.ResourceNotFound, setattr,
                          component.Component(), 'assembly_uuid', '42d')



# Copyright 2018 ZTE Corporation.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_policy import policy

from solum.common.policies import base

sensor_policies = [
    policy.DocumentedRuleDefault(
        name='get_sensors',
        check_str=base.RULE_DEFAULT,
        description='Return all sensors, based on the query provided.',
        operations=[{'path': '/v1/sensors',
                     'method': 'GET'}]),
    policy.DocumentedRuleDefault(
        name='show_sensor',
        check_str=base.RULE_DEFAULT,
        description='Return a sensor.',
        operations=[{'path': '/v1/sensors/{sensor_id}',
                     'method': 'GET'}]),
    policy.DocumentedRuleDefault(
        name='update_sensor',
        check_str=base.RULE_DEFAULT,
        description='Modify this sensor.',
        operations=[{'path': '/v1/sensors/{sensor_id}',
                     'method': 'PUT'}]),
    policy.DocumentedRuleDefault(
        name='create_sensor',
        check_str=base.RULE_DEFAULT,
        description='Create a new sensor.',
        operations=[{'path': '/v1/sensors',
                     'method': 'POST'}]),
    policy.DocumentedRuleDefault(
        name='delete_sensor',
        check_str=base.RULE_DEFAULT,
        description='Delete a sensor.',
        operations=[{'path': '/v1/sensors/{sensor_id}',
                     'method': 'DELETE'}])
]


def list_rules():
    return sensor_policies



# Copyright 2015 Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime
import json

from oslo_config import cfg
from oslo_utils import uuidutils

from solum.api.handlers import handler
from solum.common import exception
from solum.common import repo_utils
from solum import objects
from solum.objects import image
from solum.objects.sqlalchemy import workflow
from solum.worker import api as worker_api


API_SERVICE_OPTS = [
    cfg.StrOpt('image_format',
               default='qcow2',
               help='The format of the image to output'),
    cfg.StrOpt('source_format',
               default='heroku',
               help='The format of source repository'),
    cfg.StrOpt('rebuild_phrase',
               default='solum retry tests',
               help='Comment phrase to trigger rebuilding'),
    cfg.IntOpt('max_instances_per_app',
               default=100,
               help='Application scale limit'),
]


def list_opts():
    yield 'api', API_SERVICE_OPTS


CONF = cfg.CONF
CONF.register_opts(API_SERVICE_OPTS, group='api')

IMAGE_STATES = image.States


class WorkflowHandler(handler.Handler):
    """Fulfills a request on the workflow resource."""

    def _update_app_scale_config(self, app, data):

        scale_config = dict()
        target = data.get('scale_target', '1')
        try:
            target = int(target)
        except ValueError:
            msg = "Must provide integer value for scale target."
            raise exception.BadRequest(reason=msg)

        if target <= 0:
            msg = "Scale target must be greater than zero."
            raise exception.BadRequest(reason=msg)

        if target > cfg.CONF.api.max_instances_per_app:
            msg = "Target scale '%s' exceeds maximum scale limit '%s'." % (
                target, cfg.CONF.api.max_instances_per_app)
            raise exception.ResourceLimitExceeded(reason=msg)

        current_config = app.scale_config

        if current_config:
            current_config[app.name]['target'] = str(target)
            scale_config['scale_config'] = current_config
        else:
            config_data = dict()
            config_data['target'] = str(target)
            app_scale_config = dict()
            app_scale_config[app.name] = config_data
            scale_config = dict()
            scale_config['scale_config'] = app_scale_config

        objects.registry.App.update_and_save(self.context, app.id,
                                             scale_config)

    def get(self, id):
        """Return a workflow."""
        wf = objects.registry.Workflow.get_by_uuid(self.context, id)

        # Set wf status from that of the corresponding assembly's status
        assembly = objects.registry.Assembly.get_by_id(self.context,
                                                       wf.assembly)
        wf.status = assembly.status
        return wf

    def delete(self, id):
        """Delete an existing workflow."""
        db_obj = objects.registry.Workflow.get_by_uuid(self.context, id)
        db_obj.destroy(self.context)

    def create(self, data, commit_sha, status_url, du_id):
        """Create a new workflow."""

        db_obj = objects.registry.Workflow()
        db_obj.id = uuidutils.generate_uuid()
        db_obj.user_id = self.context.user
        db_obj.project_id = self.context.project_id
        db_obj.deleted = False

        db_obj.app_id = data['app_id']
        db_obj.source = data['source']
        db_obj.config = data['config']
        db_obj.actions = data['actions']

        now = datetime.datetime.utcnow()
        db_obj.created_at = now
        db_obj.updated_at = now

        app_obj = objects.registry.App.get_by_id(self.context,
                                                 db_obj.app_id)

        if str(app_obj.languagepack).lower() == 'false' and not du_id:
            msg = ("App {app} registered without a languagepack and no "
                   "du id specified. Either register the app with"
                   " a languagepack, or if there is already a pre"
                   "built du for the app, specify its id with --du-id"
                   " command line flag.")
            msg = msg.format(app=app_obj.name)
            raise exception.BadRequest(reason=msg)

        self._update_app_scale_config(app_obj, data)

        plan, assem = PlanAssemblyAdapter(self.context,
                                          db_obj,
                                          app_obj).create_dummies()

        db_obj.assembly = assem.id

        workflow.Workflow.insert(self.context, db_obj)

        self._execute_workflow_actions(db_obj, app_obj, assem,
                                       commit_sha=commit_sha,
                                       status_url=status_url,
                                       du_id=du_id)

        # TODO(devkulkarni): Update status of actions

        return db_obj

    def get_all_by_id(self, resource_uuid):
        return objects.registry.UserlogList.get_all_by_id(
            self.context, resource_uuid=resource_uuid)

    def get_all(self, app_id=None):
        """Return all of an app's workflows."""
        return objects.registry.WorkflowList.get_all(self.context,
                                                     app_id=app_id)

    def _execute_workflow_actions(self, wf_obj, app_obj, assem,
                                  verb='launch_workflow',
                                  commit_sha='', status_url=None,
                                  du_id=None):
        image = objects.registry.Image()

        image.name = app_obj.name
        image.source_uri = wf_obj.source['repository']
        image.base_image_id = app_obj.languagepack
        image.image_format = CONF.api.image_format
        image.uuid = uuidutils.generate_uuid()
        image.user_id = self.context.user
        image.project_id = self.context.project_id
        image.status = IMAGE_STATES.QUEUED
        image.create(self.context)
        test_cmd = wf_obj.config['test_cmd']
        run_cmd = wf_obj.config['run_cmd']

        ports = app_obj.ports

        app_raw_content = json.loads(app_obj.raw_content)
        if ('repo_token' in app_raw_content):
            repo_token = app_raw_content['repo_token']
        else:
            repo_token = ''

        if ('private' in wf_obj.source.keys()):
            private = wf_obj.source['private']
        else:
            private = False

        if ('private_ssh_key' in wf_obj.source.keys()):
            private_ssh_key = wf_obj.source['private_ssh_key']
        else:
            private_ssh_key = ''

        # TODO(devkulkarni): Check whether we need image.source_format
        image.source_format = 'solum'

        git_info = {
            'source_url': image.source_uri,
            'commit_sha': commit_sha,
            'repo_token': repo_token,
            'status_url': status_url,
            'private': private,
            'private_ssh_key': private_ssh_key
        }

        if test_cmd:
            repo_utils.send_status(0, status_url, repo_token, pending=True)

        worker_api.API(context=self.context).build_app(
            verb=verb,
            build_id=image.id,
            git_info=git_info,
            ports=ports,
            name=image.name,
            base_image_id=image.base_image_id,
            source_format=image.source_format,
            image_format=image.image_format,
            assembly_id=assem.id,
            workflow=assem.workflow,
            test_cmd=test_cmd,
            run_cmd=run_cmd,
            du_id=du_id)


class PlanAssemblyAdapter(object):
    """Plan and assembly adapter."""
    # Adapter that we use to create assembly and plan
    # objects. This is intended to smooth the transition
    # to the new app and workflow models without causing
    # backwards breaking changes.

    # 1) We need to create Assembly object in our database.
    # This is because,
    # solum/worker/handlers/shell.py and
    # solum/deployer/handlers/heat.py
    # heavily reference 'assembly' (and its attributes)

    # 2) We need to create Plan object because Assembly
    # has a foreign key reference to plan object
    def __init__(self, context, wf_obj, app_obj):
        self.context = context
        self.wf_obj = wf_obj
        self.app_obj = app_obj

    def create_dummies(self):
        # Create only one plan
        try:
            plan = objects.registry.Plan.get_by_uuid(self.context,
                                                     self.app_obj.id)
        except exception.ResourceNotFound:
            plan = objects.registry.Plan()
            plan.uuid = self.app_obj.id
            plan.user_id = self.context.user
            plan.project_id = self.context.project_id
            plan.name = self.app_obj.name
            plan.create(self.context)

        assembly = objects.registry.Assembly()
        assembly.plan_id = plan.id
        assembly.user_id = self.context.user
        assembly.project_id = self.context.project_id
        assembly.name = self.app_obj.name
        assembly.uuid = uuidutils.generate_uuid()

        assembly.workflow = self.wf_obj.actions

        assembly.create(self.context)

        return plan, assembly



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from solum.i18n import _
from solum.tests import base
from solum.tests import utils
from solum.tests.worker.handlers import test_shell
from solum.worker.handlers import noop as noop_handler


class HandlerTest(base.BaseTestCase):
    def setUp(self):
        super(HandlerTest, self).setUp()
        self.ctx = utils.dummy_context()

    @mock.patch('solum.worker.handlers.noop.LOG')
    def test_echo(self, fake_LOG):
        noop_handler.Handler().echo({}, 'foo')
        fake_LOG.debug.assert_called_once_with(_('%s') % 'foo')

    @mock.patch('solum.worker.handlers.noop.LOG')
    def test_build(self, fake_LOG):
        git_info = test_shell.mock_git_info()
        args = [5, git_info, 'new_app', '1-2-3-4', 'heroku', 'docker',
                44, None, None]
        noop_handler.Handler().build(self.ctx, *args)
        message = 'Build ' + ', '.join([str(a) for a in args])
        fake_LOG.debug.assert_called_once_with(_("%s") % message)

    @mock.patch('solum.worker.handlers.noop.LOG')
    def test_unittest(self, fake_LOG):
        git_info = test_shell.mock_git_info()
        args = [5, git_info, 'new_app',
                '1-2-3-4', 'heroku', 'docker', 44, 'pep8']
        noop_handler.Handler().unittest(self.ctx, *args)
        message = 'Unittest ' + ', '.join([str(a) for a in args])
        fake_LOG.debug.assert_called_once_with(_("%s") % message)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
import wsme
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import assembly
import solum.api.controllers.v1.userlog as userlog_controller
from solum.api.handlers import assembly_handler
from solum.common import exception
from solum.common import policy
from solum.common import request
from solum.i18n import _
from solum import objects


class AssemblyController(rest.RestController):
    """Manages operations on a single assembly."""

    def __init__(self, assembly_id):
        super(AssemblyController, self).__init__()
        self._id = assembly_id

    @pecan.expose()
    def _lookup(self, primary_key, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        if primary_key == 'logs':
            logs = userlog_controller.UserlogsController(self._id)
            return logs, remainder

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(assembly.Assembly)
    def get(self):
        """Return this assembly."""
        policy.check('show_assembly',
                     pecan.request.security_context)
        request.check_request_for_https()
        handler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return assembly.Assembly.from_db_model(handler.get(self._id), host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(assembly.Assembly, body=assembly.Assembly)
    def put(self, data):
        """Modify this assembly."""
        policy.check('update_assembly',
                     pecan.request.security_context)
        handler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        res = handler.update(self._id,
                             data.as_dict(objects.registry.Assembly))
        host_url = pecan.request.application_url.rstrip('/')
        return assembly.Assembly.from_db_model(res, host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(status_code=204)
    def delete(self):
        """Delete this assembly."""
        policy.check('delete_assembly',
                     pecan.request.security_context)
        handler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        return handler.delete(self._id)


class AssembliesController(rest.RestController):
    """Manages operations on the assemblies collection."""

    @pecan.expose()
    def _lookup(self, assembly_id, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return AssemblyController(assembly_id), remainder

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose(assembly.Assembly, body=assembly.Assembly,
                         status_code=201)
    def post(self, data):
        """Create a new assembly."""
        policy.check('create_assembly', pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        js_data = data.as_dict(objects.registry.Assembly)
        if data.plan_uri is not wsme.Unset:
            plan_uri = data.plan_uri
            if plan_uri.startswith(host_url):
                pl_uuid = plan_uri.split('/')[-1]
                pl = objects.registry.Plan.get_by_uuid(
                    pecan.request.security_context, pl_uuid)
                js_data['plan_id'] = pl.id
            else:
                # TODO(asalkeld) we are not hosting the plan so
                # download the plan and insert it into our db.
                raise exception.BadRequest(reason=_(
                    'The plan was not hosted in solum'))

        if js_data.get('plan_id') is None:
            raise exception.BadRequest(reason=_(
                'The plan was not given or could not be found'))

        handler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        return assembly.Assembly.from_db_model(
            handler.create(js_data), host_url)

    @exception.wrap_wsme_pecan_controller_exception
    @wsme_pecan.wsexpose([assembly.Assembly])
    def get_all(self):
        """Return all assemblies, based on the query provided."""
        policy.check('get_assemblies',
                     pecan.request.security_context)
        request.check_request_for_https()
        handler = assembly_handler.AssemblyHandler(
            pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        return [assembly.Assembly.from_db_model(assm, host_url)
                for assm in handler.get_all()]



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import core
from pecan import rest
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.camp.v1_1.datamodel import (attribute_definitions
                                                       as model)
from solum.api.handlers.camp import attribute_definition_handler
from solum.common import exception


class AttributeDefinitionsController(rest.RestController):
    """CAMP v1.1 attribute_definitions controller."""

    @exception.wrap_wsme_controller_exception
    @wsme_pecan.wsexpose(model.AttributeDefinition, wtypes.text)
    def get_one(self, attr_def_name):
        handler = (attribute_definition_handler.
                   AttributeDefinitionHandler(pecan.request.security_context))
        raw_def = handler.get(attr_def_name)
        if not raw_def:
            core.abort(404,
                       '%s is not a attribute_definition' %
                       attr_def_name)
        host_url = pecan.request.application_url.rstrip('/')
        return raw_def.fix_uris(host_url)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

ASSEM_URI_STR = '%s/camp/v1_1/assemblies/%s'

ASSEMS_URI_STR = '%s/camp/v1_1/assemblies'

ATTRIBUTE_DEF_URI_STR = '%s/camp/v1_1/attribute_definitions/%s'

DEPLOY_PARAMS_URI = '%s/camp/v1_1/parameter_definitions/deploy_params'

EXTN_URI_STR = '%s/v1/extensions/%s'

EXTNS_URI_STR = '%s/camp/v1_1/extensions'

FORMAT_URI_STR = '%s/camp/v1_1/formats/%s'

FORMATS_URI_STR = '%s/camp/v1_1/formats'

JSON_FORMAT_URI_STR = '%s/camp/v1_1/formats/json_format'
JSON_FORMAT_NAME_STR = 'JSON'

PARAM_DEF_URI_STR = '%s/camp/v1_1/parameter_definition/%s'
PARAM_DEFS_URI_STR = '%s/camp/v1_1/parameter_definitions/%s'

PLAN_URI_STR = '%s/camp/v1_1/plans/%s'

PLANS_URI_STR = '%s/camp/v1_1/plans'

PLATFORM_URI_STR = '%s/camp/v1_1/platform'

SERV_URI_STR = '%s/v1/services/%s'

SERVS_URI_STR = '%s/camp/v1_1/services'

TYPE_DEF_URI_STR = '%s/camp/v1_1/type_definitions/%s'
TYPE_DEFS_URI_STR = '%s/camp/v1_1/type_definitions'



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from oslo_config import fixture as config

from solum.api import auth
from solum.common import context
from solum.tests import base
from solum.tests import fakes


@mock.patch('solum.api.auth.AuthProtocolWrapper',
            new_callable=fakes.FakeAuthProtocol)
class TestAuth(base.BaseTestCase):

    def setUp(self):
        super(TestAuth, self).setUp()
        self.CONF = self.useFixture(config.Config())
        self.app = fakes.FakeApp()

    def test_check_auth_option_enabled(self, mock_auth):

        self.CONF.config(auth_protocol="http",
                         auth_version="v2.0",
                         www_authenticate_uri=None,
                         group=auth.OPT_GROUP_NAME)
        self.CONF.config(enable_authentication=True)
        result = auth.install(self.app, self.CONF.conf)
        self.assertIsInstance(result, fakes.FakeAuthProtocol)

    def test_check_auth_option_disabled(self, mock_auth):
        self.CONF.config(auth_protocol="http",
                         auth_version="v2.0",
                         www_authenticate_uri=None,
                         group=auth.OPT_GROUP_NAME)
        self.CONF.config(enable_authentication=False)
        result = auth.install(self.app, self.CONF.conf)
        self.assertIsInstance(result, fakes.FakeApp)

    def test_auth_hook_before_method(self, mock_cls):
        state = mock.Mock(request=fakes.FakePecanRequest())
        hook = auth.ContextHook()
        hook.before(state)
        ctx = state.request.security_context
        self.assertIsInstance(ctx, context.RequestContext)
        self.assertEqual(ctx.auth_token,
                         fakes.fakeAuthTokenHeaders['X-Auth-Token'])
        self.assertEqual(ctx.project_id,
                         fakes.fakeAuthTokenHeaders['X-Project-Id'])
        self.assertEqual(ctx.user,
                         fakes.fakeAuthTokenHeaders['X-User-Id'])
        self.assertEqual(ctx.roles,
                         [u'admin', u'ResellerAdmin', u'_member_'])
        self.assertEqual(ctx.user_name,
                         fakes.fakeAuthTokenHeaders['X-User-Name'])
        self.assertEqual(ctx.domain,
                         fakes.fakeAuthTokenHeaders['X-Domain-Name'])
        self.assertEqual(ctx.project_domain,
                         fakes.fakeAuthTokenHeaders['X-Project-Domain-Id'])
        self.assertEqual(ctx.user_domain,
                         fakes.fakeAuthTokenHeaders['X-User-Domain-Id'])
        self.assertIsNone(ctx.auth_token_info)

    def test_auth_hook_before_method_auth_info(self, mock_cls):
        state = mock.Mock(request=fakes.FakePecanRequest())
        state.request.environ['keystone.token_info'] = 'assert_this'
        hook = auth.ContextHook()
        hook.before(state)
        ctx = state.request.security_context
        self.assertIsInstance(ctx, context.RequestContext)
        self.assertEqual(fakes.fakeAuthTokenHeaders['X-Auth-Token'],
                         ctx.auth_token)
        self.assertEqual('assert_this', ctx.auth_token_info)



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sqlalchemy.orm import exc
from unittest import mock

from solum import objects
from solum.objects import registry
from solum.objects.sqlalchemy import image
from solum.tests import base
from solum.tests import fakes
from solum.tests import utils


class TestImage(base.BaseTestCase):
    def setUp(self):
        super(TestImage, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()
        self.data = [{'project_id': self.ctx.project_id,
                      'user_id': 'fred',
                      'uuid': '25f7fa50-b980-4452-a550-dea5fd98ffc2',
                      'name': 'image1',
                      'description': 'test image'}]
        utils.create_models_from_data(image.Image, self.data, self.ctx)

    def test_objects_registered(self):
        self.assertTrue(registry.Image)
        self.assertTrue(registry.ImageList)

    def test_get_all(self):
        lst = image.ImageList()
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data(self):
        test_srvc = image.Image().get_by_id(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(test_srvc, key))

    @mock.patch('solum.objects.registry.Image.get_by_name')
    def test_get_lp_by_name_or_uuid_with_name(self, mock_img):
        mock_img.return_value = fakes.FakeImage()
        img = image.Image()
        img.get_lp_by_name_or_uuid(self.ctx, self.data[0]['name'])
        mock_img.assert_called_once_with(self.ctx,
                                         self.data[0]['name'],
                                         False)

    @mock.patch('sqlalchemy.orm.query.Query.one')
    @mock.patch('solum.objects.registry.Image.get_by_name')
    def test_get_lp_by_name_or_uuid_with_uuid(self, mock_img, mock_query):
        mock_img.return_value = fakes.FakeImage()
        mock_query.side_effect = exc.NoResultFound()
        image.Image().get_lp_by_name_or_uuid(self.ctx, self.data[0]['uuid'])
        mock_img.assert_called_once_with(self.ctx,
                                         self.data[0]['uuid'],
                                         False)


class TestStates(base.BaseTestCase):
    def test_as_dict(self):
        self.assertEqual(objects.image.States.as_dict(),
                         {'BUILDING': 'BUILDING', 'READY': 'READY',
                          'ERROR': 'ERROR', 'QUEUED': 'QUEUED'})

    def test_values(self):
        self.assertEqual(sorted(objects.image.States.values()),
                         sorted(['BUILDING', 'READY', 'QUEUED', 'ERROR']))



# Copyright 2014 - Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils
import sqlalchemy as sa
from sqlalchemy.orm import exc

from solum.objects import image as abstract
from solum.objects.sqlalchemy import models as sql

cfg.CONF.import_opt('operator_project_id',
                    'solum.api.handlers.language_pack_handler',
                    group='api')

operator_id = cfg.CONF.api.operator_project_id

LOG = logging.getLogger(__name__)


class Image(sql.Base, abstract.Image):
    """Represent a image in sqlalchemy."""

    __tablename__ = 'image'
    __resource__ = 'images'
    __table_args__ = sql.table_args()

    id = sa.Column(sa.Integer, primary_key=True, autoincrement=True)
    uuid = sa.Column(sa.String(36), nullable=False)
    name = sa.Column(sa.String(100))
    source_uri = sa.Column(sa.String(1024))
    source_format = sa.Column(sa.String(36))
    description = sa.Column(sa.String(255))
    project_id = sa.Column(sa.String(36))
    user_id = sa.Column(sa.String(36))
    tags = sa.Column(sa.Text)
    status = sa.Column(sa.String(12))
    base_image_id = sa.Column(sa.String(36))
    created_image_id = sa.Column(sa.String(36))
    image_format = sa.Column(sa.String(12))
    artifact_type = sa.Column(sa.String(36))
    external_ref = sa.Column(sa.String(1024))
    docker_image_name = sa.Column(sa.String(512))

    @classmethod
    def get_num_of_lps(cls, context):
        try:
            session = Image.get_session()
            oper_result = session.query(cls).filter_by(
                project_id=context.project_id, status='READY')
            cnt = oper_result.count()
            return cnt
        except exc.NoResultFound:
            LOG.debug("Exception encountered in getting count of number"
                      " of languagepacks")

    @classmethod
    def get_lp_by_name_or_uuid(cls, context, name_or_uuid,
                               include_operators_lp=False):
        if uuidutils.is_uuid_like(name_or_uuid):
            try:
                session = Image.get_session()
                result = session.query(cls).filter_by(
                    artifact_type='language_pack', uuid=name_or_uuid)
                if include_operators_lp is True:
                    project_id = context.project_id
                    result = result.filter(
                        Image.project_id.in_([operator_id, project_id]))
                    return result.one()
                else:
                    return sql.filter_by_project(context, result).one()
            except exc.NoResultFound:
                return cls.get_by_name(context, name_or_uuid,
                                       include_operators_lp)
        else:
            return cls.get_by_name(context, name_or_uuid, include_operators_lp)

    @classmethod
    def get_by_name(cls, context, name, include_operators_lp=False):
        try:
            session = Image.get_session()
            result = session.query(cls).filter_by(
                artifact_type='language_pack', name=name)
            if include_operators_lp is True:
                result = result.filter(
                    Image.project_id.in_([operator_id, context.project_id]))
                return result.one()
            else:
                return sql.filter_by_project(context, result).one()
        except exc.NoResultFound:
            cls._raise_not_found(name)

    @classmethod
    def get_all_languagepacks(cls, context):
        """Return all images that are languagepacks."""
        session = Image.get_session()
        result = session.query(cls)
        result = result.filter_by(artifact_type='language_pack')
        result = sql.filter_by_project(context, result)

        # Include Languagepacks that have been created by the operator, and
        # are in the 'READY' state.
        # The operator LP is identified based on the operator_project_id
        # config setting in solum.conf
        oper_result = session.query(cls)
        oper_result = oper_result.filter_by(artifact_type='language_pack')
        oper_result = oper_result.filter_by(status='READY')
        oper_result = oper_result.filter_by(project_id=operator_id)

        return result.union(oper_result).all()


class ImageList(abstract.ImageList):
    """Represent a list of images in sqlalchemy."""

    @classmethod
    def get_all(cls, context):
        """Return all images."""
        return ImageList(sql.model_query(context, Image))

    @classmethod
    def get_all_languagepacks(cls, context):
        """Return all images that are languagepacks."""
        return ImageList(sql.model_query(
            context, Image).filter_by(artifact_type='language_pack'))



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add docker_image_name to image table

Revision ID: 452f34e8ea3
Revises: 5583c6e70156
Create Date: 2015-04-02 22:07:34.346737

"""
from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '452f34e8ea3'
down_revision = '5583c6e70156'


def upgrade():
    op.add_column('image', sa.Column('docker_image_name',
                                     sa.String(length=512)))


def downgrade():
    op.drop_column('image', 'docker_image_name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from solum.api.controllers.camp.v1_1.datamodel import (parameter_definitions
                                                       as pd)
from solum.api.handlers import handler


DEPLOY_PARAM_DESCRIPTION = ("Solum CAMP API definitions of the 'pdp_uri', "
                            "'plan_uri', 'pdp_file', and 'plan_file' "
                            "parameters.")

DEPLOY_PARAM_LINKS = [
    pd.ParameterDefinitionLink(href='pdp_uri_param',
                               target_name='pdp_uri',
                               required=False),
    pd.ParameterDefinitionLink(href='plan_uri_param',
                               target_name='plan_uri',
                               required=False),
    pd.ParameterDefinitionLink(href='pdp_file_param',
                               target_name="pdp_file",
                               required=False),
    pd.ParameterDefinitionLink(href='plan_file_param',
                               target_name="plan_file",
                               required=False)
]

NDT_PARAM_DESCRIPTION = ("Solum CAMP API definitions of the 'name', "
                         "'description', and 'tag' parameters.")

NDT_PARAM_LINKS = [
    pd.ParameterDefinitionLink(href='name_param',
                               target_name='name',
                               required=False),
    pd.ParameterDefinitionLink(href='description_param',
                               target_name='description',
                               required=False),
    pd.ParameterDefinitionLink(href='tags_param',
                               target_name='tags',
                               required=False)
]

GLOBAL_PARAM_DEFS = {
    'deploy_params':
        pd.ParameterDefinitions(uri='deploy_params',
                                name='Solum_CAMP_deploy_parameters',
                                type='parameter_definitions',
                                description=DEPLOY_PARAM_DESCRIPTION,
                                parameter_definition_links=DEPLOY_PARAM_LINKS),
    'ndt_params':
        pd.ParameterDefinitions(uri='ndt_params',
                                name='Solum_CAMP_NDT_parameters',
                                type='parameter_definitions',
                                description=NDT_PARAM_DESCRIPTION,
                                parameter_definition_links=NDT_PARAM_LINKS)
}


class ParameterDefinitionsHandler(handler.Handler):

    def get(self, path):
        if path in GLOBAL_PARAM_DEFS:
            return GLOBAL_PARAM_DEFS[path]



# Copyright 2015 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
import pecan
from pecan import rest
import wsmeext.pecan as wsme_pecan

from solum.api.controllers.v1.datamodel import workflow
from solum.api.controllers.v1 import userlog as userlog_controller
from solum.api.handlers import app_handler
from solum.api.handlers import workflow_handler as wf_handler
from solum.common import clients
from solum.common import exception
from solum.common import request

cfg.CONF.import_opt('image_storage', 'solum.worker.config', group='worker')


class WorkflowController(rest.RestController):
    """Manages operations on a single workflow."""

    def __init__(self, app_id, wf_id):
        super(WorkflowController, self).__init__(self)
        self.app_id = app_id
        self.wf_id = wf_id

    @pecan.expose()
    def _lookup(self, primary_key, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        if primary_key == 'logs':
            logs = userlog_controller.UserlogsController(self.wf_id)
            return logs, remainder

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose(workflow.Workflow)
    def get(self):
        """Return this workflow."""
        request.check_request_for_https()
        handler = wf_handler.WorkflowHandler(pecan.request.security_context)
        wf_model = handler.get(self.wf_id)
        host_url = pecan.request.application_url.rstrip('/')
        wf_model = workflow.Workflow.from_db_model(wf_model, host_url)
        return wf_model


class WorkflowsController(rest.RestController):
    """Manages operations on all of an app's workflows."""

    def __init__(self, app_id):
        super(WorkflowsController, self).__init__(self)
        self.app_id = app_id

    @pecan.expose()
    def _lookup(self, wf_uuid, *remainder):
        if remainder and not remainder[-1]:
            remainder = remainder[:-1]
        return WorkflowController(self.app_id, wf_uuid), remainder

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose(workflow.Workflow, body=workflow.Workflow,
                         status_code=200)
    def post(self, data):
        """Create a new workflow."""
        request.check_request_for_https()
        if not data:
            raise exception.BadRequest(reason='No data.')

        ahandler = app_handler.AppHandler(pecan.request.security_context)
        app_model = ahandler.get(self.app_id)

        handler = wf_handler.WorkflowHandler(pecan.request.security_context)

        data.app_id = app_model.id
        data.config = app_model.workflow_config
        data.source = app_model.source

        wf_data = data.as_dict(workflow.Workflow)

        du_id = None
        if data.du_id:
            du_id = data.du_id
            self._verify_du_exists(pecan.request.security_context, du_id)

        host_url = pecan.request.application_url.rstrip('/')
        return workflow.Workflow.from_db_model(handler.create(wf_data,
                                                              commit_sha='',
                                                              status_url='',
                                                              du_id=du_id),
                                               host_url)

    @exception.wrap_pecan_controller_exception
    @wsme_pecan.wsexpose([workflow.Workflow])
    def get_all(self):
        """Return all of one app's workflows, based on the query provided."""
        request.check_request_for_https()
        handler = wf_handler.WorkflowHandler(pecan.request.security_context)
        host_url = pecan.request.application_url.rstrip('/')
        all_wfs = [workflow.Workflow.from_db_model(obj, host_url)
                   for obj in handler.get_all(app_id=self.app_id)]
        return all_wfs

    def _verify_du_exists(self, ctxt, du_id):
        du_image_backend = cfg.CONF.worker.image_storage
        if du_image_backend.lower() == 'glance':
            self._verify_du_image_exists_in_glance(ctxt, du_id)
        elif du_image_backend.lower() == 'swift':
            self._verify_du_image_exists_in_swift(ctxt, du_id)
        else:
            raise exception.BadRequest(message="DU image id not recognized.")
        return

    def _verify_du_image_exists_in_glance(self, ctxt, du_id):
        osc = clients.OpenStackClients(ctxt)
        osc.glance().images.get(du_id)
        return

    def _verify_du_image_exists_in_swift(self, du_id):
        # TODO(devkulkarni): Check if specified du_id exists in swift
        return



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""WSGI script for solum-api.

Script for running solum-api under Apache2.
"""

from oslo_config import cfg
import oslo_i18n as i18n
from oslo_log import log as logging

from solum.api import app as api_app
from solum.common import config
from solum import objects


def init_application():
    i18n.enable_lazy()

    LOG = logging.getLogger('solum.api')

    logging.register_options(cfg.CONF)
    cfg.CONF(project='solum')
    logging.setup(cfg.CONF, 'solum')

    config.set_config_defaults()
    objects.load()

    port = cfg.CONF.api.port
    host = cfg.CONF.api.host
    LOG.info(('Starting Solum REST API on %(host)s:%(port)s'),
             {'host': host, 'port': port})
    return api_app.setup_app()



# Copyright 2014 - Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect

from oslo_config import cfg
from oslo_context import context

AUTH_OPTS = [
    cfg.StrOpt('solum_admin_tenant_id',
               default='',
               help='Tenant id of global admin'),
]


def list_opts():
    yield None, AUTH_OPTS


CONF = cfg.CONF
CONF.register_opts(AUTH_OPTS)


class RequestContext(context.RequestContext):
    def __init__(self, auth_token=None, user=None, project_id=None,
                 domain=None, user_domain=None, project_domain=None,
                 is_admin=False, read_only=False, request_id=None,
                 user_name=None, roles=None, auth_url=None, trust_id=None,
                 auth_token_info=None, password=None, project_name=None):
        super(RequestContext, self).__init__(auth_token=auth_token,
                                             user=user,
                                             project_id=project_id,
                                             domain=domain,
                                             user_domain=user_domain,
                                             project_domain=project_domain,
                                             is_admin=is_admin,
                                             read_only=read_only,
                                             show_deleted=False,
                                             request_id=request_id,
                                             roles=roles)
        self.user_name = user_name
        self.auth_url = auth_url
        self.trust_id = trust_id
        self.auth_token_info = auth_token_info
        self.password = password
        self.project_name = project_name
        global_admin_id = CONF.get('solum_admin_tenant_id')
        if global_admin_id and global_admin_id == project_id:
            self.is_admin = True

    def to_dict(self):
        values = super(RequestContext, self).to_dict()
        values.update({
            'user_name': self.user_name,
            'auth_url': self.auth_url,
            'auth_token_info': self.auth_token_info,
            'trust_id': self.trust_id})
        return values

    @classmethod
    def from_dict(cls, values):
        allowed = [arg for arg in
                   inspect.getargspec(RequestContext.__init__).args
                   if arg != 'self']
        kwargs = dict((k, v) for (k, v) in values.items() if k in allowed)
        return cls(**kwargs)



# Copyright 2015 Rackspace Hosting
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from oslo_utils import uuidutils

import json

from solum.api.handlers import common
from solum.api.handlers import handler
from solum.api.handlers import workflow_handler
from solum.common import exception
from solum.common import keystone_utils
from solum.common import utils
from solum.deployer import api as deploy_api
from solum import objects


LOG = logging.getLogger(__name__)


class AppHandler(handler.Handler):
    """Fulfills a request on the app resource."""

    def get(self, id):
        """Return an app."""
        return objects.registry.App.get_by_uuid(self.context, id)

    def patch(self, id, data):
        """Update an app."""
        db_obj = objects.registry.App.get_by_uuid(self.context, id)

        data_dict = data.as_dict(objects.registry.App)
        obj_dict = db_obj.as_dict()

        # Source and workflow are a little tricky to update.
        new_source = obj_dict['source']
        new_source.update(data_dict.get('source', {}))
        data_dict['source'] = new_source

        new_wf = obj_dict['workflow_config']
        new_wf.update(data_dict.get('workflow_config', {}))
        data_dict['workflow_config'] = new_wf

        if data_dict.get('repo_token'):
            new_raw_content = json.loads(obj_dict['raw_content'])
            new_raw_content['repo_token'] = data_dict['repo_token']
            data_dict['raw_content'] = json.dumps(new_raw_content)
            del data_dict['repo_token']

        updated = objects.registry.App.update_and_save(self.context,
                                                       id, data_dict)
        return updated

    def delete(self, id):
        """Delete an existing app."""
        try:
            objects.registry.App.get_by_uuid(self.context, id)
        except exception.ResourceNotFound:
            raise

        deploy_api.API(context=self.context).destroy_app(id)

    def create(self, data):
        """Create a new app."""
        db_obj = objects.registry.App()
        db_obj.id = uuidutils.generate_uuid()
        db_obj.user_id = self.context.user
        db_obj.project_id = self.context.project_id
        db_obj.deleted = False

        # create a delegation trust_id\token, if required
        db_obj.trust_id = keystone_utils.create_delegation_token(self.context)

        db_obj.trust_user = self.context.user_name

        db_obj.name = data.get('name')
        db_obj.description = data.get('description')
        db_obj.languagepack = data.get('languagepack')
        db_obj.stack_id = data.get('stack_id')
        db_obj.ports = data.get('ports')
        db_obj.source = data.get('source')
        db_obj.workflow_config = data.get('workflow_config')
        db_obj.trigger_uuid = uuidutils.generate_uuid()
        db_obj.trigger_actions = data.get('trigger_actions')

        raw_content = data.get('raw_content')
        raw_content_json = json.loads(raw_content)

        raw_content_json['username'] = self.context.user_name
        raw_content_json['user_id'] = self.context.user
        encrypted_password = utils.encrypt(self.context.password)
        # encrypted_password contains encoded characters that cannot
        # be json dumped which is required in order to save it to db.
        # So we need to decode it first.
        decoded_password = encrypted_password.decode('ISO-8859-1')
        raw_content_json['password'] = decoded_password

        raw_content_json['auth_url'] = self.context.auth_url
        raw_content_json['tenant'] = self.context.project_id
        raw_content_json['tenant_name'] = self.context.project_name

        db_obj.raw_content = json.dumps(raw_content_json)

        common.check_url(db_obj.source['repository'])

        db_obj.create(self.context)
        return db_obj

    def trigger_workflow(self, trigger_id, commit_sha='',
                         status_url=None, collab_url=None, workflow=None):
        """Get trigger by trigger id and start git workflow associated."""
        # Note: self.context will be None at this point as this is a
        # non-authenticated request.
        app_obj = objects.registry.App.get_by_trigger_id(None, trigger_id)
        # get the trust context and authenticate it.
        try:
            self.context = keystone_utils.create_delegation_context(
                app_obj, self.context)
            self.context.project_id = app_obj.project_id
            self.context.user = app_obj.user_id
            self.context.user_name = app_obj.trust_user
        except exception.AuthorizationFailure as auth_ex:
            LOG.warning(auth_ex)
            return

        # TODO(devkulkarni): Call repo_utils.verify_artifact
        # as we are calling it in the plan_handler to verify
        # the collaborator
        self._build_artifact(app_obj, commit_sha=commit_sha,
                             status_url=status_url, wf=workflow)

    def _build_artifact(self, app, commit_sha='',
                        status_url=None, wf=None):

        # TODO(devkulkarni): Check why wf is being passed
        # if wf is None:
        #    wf = ['unittest', 'build', 'deploy']
        wfhand = workflow_handler.WorkflowHandler(self.context)

        wfdata = {
            'app_id': app.id,
            'name': "%s" % app.name,
            'description': '',
            'source': app.source,
            'config': app.workflow_config,
            'actions': app.trigger_actions
        }
        wfhand.create(wfdata, commit_sha=commit_sha, status_url=status_url,
                      du_id=None)

    def get_all(self):
        """Return all apps."""
        all_apps = objects.registry.AppList.get_all(self.context)
        return all_apps



# -*- encoding: utf-8 -*-
# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock
import uuid

from oslo_config import cfg
import wsme

from solum.common import exception
from solum.tests import base


class ExceptionTestCase(base.BaseTestCase):
    """Test cases for exception code."""

    def test_with_kwargs(self):
        exc = exception.ResourceNotFound(name='application', id='green_paint')
        self.assertIn('green_paint could not be found.', str(exc))
        self.assertEqual(404, exc.code)

    def test_with_kwargs_ru(self):
        exc = exception.ResourceNotFound(name='application',
                                         id=u'зеленой_краской')
        self.assertIn(u'зеленой_краской could not be found', str(exc))
        self.assertEqual(404, exc.code)

    def test_bad_kwargs_exception(self):
        cfg.CONF.set_override('fatal_exception_format_errors', True)
        self.assertRaises(KeyError,
                          exception.ResourceNotFound, a_field='green')

    def test_bad_kwargs(self):
        cfg.CONF.set_override('fatal_exception_format_errors', False)
        exc = exception.ResourceNotFound(a_field='green')
        self.assertIn('An unknown exception occurred', str(exc))
        self.assertEqual(404, exc.code)

    def test_resource_exists(self):
        exc = exception.ResourceExists(name='test')
        self.assertIn("The test resource already exists.", str(exc))
        self.assertEqual(409, exc.code)

    def test_application_exists(self):
        exc = exception.ResourceExists(name='test')
        self.assertIn("The test resource already exists.", str(exc))
        self.assertEqual(409, exc.code)

    def test_not_implemented(self):
        exc = exception.NotImplemented()
        self.assertIn("The requested operation is not implemented.", str(exc))
        self.assertEqual(501, exc.code)

    def test_wrap_controller_exception_with_server_error(self):
        exception.LOG.error = mock.Mock()

        def error_func():
            raise exception.NotImplemented()

        correlation_id = None
        try:
            exception.wrap_wsme_controller_exception(error_func)()
        except wsme.exc.ClientSideError as e:
            correlation_id = str(e).split(":")[1].strip()

        self.assertIsNotNone(correlation_id)
        self.assertIsInstance(uuid.UUID(correlation_id), uuid.UUID)
        self.assertTrue(exception.LOG.error.called)

        (args, kargs) = exception.LOG.error.call_args

        self.assertIn(correlation_id, args)
        self.assertIn(correlation_id, str(args[0] % args[1:]))

    def test_wrap_controller_exception_with_client_error(self):
        error_args = dict(reason="foo")
        expected_error_msg = str(exception.BadRequest.msg_fmt % error_args)

        def error_func():
            raise exception.BadRequest(**error_args)

        try:
            exception.wrap_wsme_controller_exception(error_func)()
            self.assertTrue(False)
        except wsme.exc.ClientSideError as e:
            self.assertEqual(expected_error_msg, e.msg)

    def test_wrap_controller_exception_with_uncatched_error(self):
        exception.LOG.error = mock.Mock()

        def error_func():
            value_error = ValueError('Hey!')
            value_error.code = 500
            raise value_error

        correlation_id = None
        try:
            exception.wrap_wsme_controller_exception(error_func)()
        except wsme.exc.ClientSideError as e:
            correlation_id = str(e).split(":")[1].strip()

        self.assertIsNotNone(correlation_id)
        self.assertIsInstance(uuid.UUID(correlation_id), uuid.UUID)
        self.assertTrue(exception.LOG.error.called)

        (args, kargs) = exception.LOG.error.call_args

        self.assertIn(correlation_id, args)
        self.assertIn(correlation_id, str(args[0] % args[1:]))



# Copyright 2013 - Noorul Islam K M
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
import pecan.testing

from solum.tests import base

__all__ = ['FunctionalTest']


class FunctionalTest(base.BaseTestCase):
    """Used for functional tests.

    Where you need to test your literal application and its
    integration with the framework.
    """

    def setUp(self):
        super(FunctionalTest, self).setUp()
        self.addCleanup(pecan.set_config, {}, overwrite=True)
        self.app = pecan.testing.load_test_app({
            'app': {
                'root': 'solum.api.controllers.root.RootController',
                'modules': ['solum.api'],
                'debug': False
            }
        })

    def assertResourceNotFound(self, url):
        resp = self.app.get(url, status=404,
                            headers={'Accept': 'application/json'})
        self.assertEqual(404, resp.status_int)



# Copyright 2013 - Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock

from oslo_config import cfg

from solum.api.controllers.v1 import trigger
from solum.tests import base
from solum.tests import fakes


@mock.patch('pecan.request', new_callable=fakes.FakePecanRequest)
@mock.patch('pecan.response', new_callable=fakes.FakePecanResponse)
@mock.patch('solum.api.controllers.v1.trigger.app_handler'
            '.AppHandler')
class TestTriggerController(base.BaseTestCase):

    def test_trigger_get_workflow_with_empty_body(self, assem_mock,
                                                  resp_mock, request_mock):
        obj = trigger.TriggerController()
        workflow = obj._get_workflow({})
        self.assertIsNone(workflow)

    def test_trigger_get_workflow_with_deploy(self, assem_mock,
                                              resp_mock, request_mock):
        obj = trigger.TriggerController()
        query = {'workflow': 'deploy'}
        workflow = obj._get_workflow(query)
        self.assertEqual(['deploy'], list(workflow))

    def test_trigger_get_workflow_with_build_deploy(self, assem_mock,
                                                    resp_mock, request_mock):
        obj = trigger.TriggerController()
        query = {'workflow': 'build+deploy'}
        workflow = obj._get_workflow(query)
        self.assertEqual(['build', 'deploy'], list(workflow))

    def test_trigger_get_workflow_with_all(self, assem_mock,
                                           resp_mock, request_mock):
        obj = trigger.TriggerController()
        query = {'workflow': 'unittest+build+deploy'}
        workflow = obj._get_workflow(query)
        self.assertEqual(['unittest', 'build', 'deploy'], list(workflow))

    def test_trigger_get_workflow_with_invalid_stage(self, assem_mock,
                                                     resp_mock, request_mock):
        obj = trigger.TriggerController()
        query = {'workflow': 'unittest+unitunitunittest'}
        workflow = obj._get_workflow(query)
        self.assertEqual(['unittest'], list(workflow))

    def test_trigger_process_request_private_repo(self, assem_mock,
                                                  resp_mock, request_mock):
        cfg.CONF.api.rebuild_phrase = "solum retry tests"
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        collab_url = ('https://api.github.com/repos/u/r/' +
                      'collaborators{/collaborator}')
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'comment': {'commit_id': 'asdf',
                                 'body': '  SOLUM retry tests ',
                                 'user': {'login': 'u'}},
                     'repository': {'statuses_url': status_url,
                                    'collaborators_url': collab_url,
                                    'private': True}}
        obj = trigger.TriggerController()
        commit_sha, collab_url = obj._process_request(body_dict)
        self.assertIsNone(collab_url)
        self.assertEqual('asdf', commit_sha)

    def test_trigger_process_request_on_valid_pub_repo(self,
                                                       assem_mock, resp_mock,
                                                       request_mock):
        cfg.CONF.api.rebuild_phrase = "solum retry tests"
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        collab_url = ('https://api.github.com/repos/u/r/' +
                      'collaborators{/collaborator}')
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'comment': {'commit_id': 'asdf',
                                 'body': 'solum retry tests',
                                 'user': {'login': 'u'}},
                     'repository': {'statuses_url': status_url,
                                    'collaborators_url': collab_url,
                                    'private': False}}
        obj = trigger.TriggerController()
        commit_sha, collab_url = obj._process_request(body_dict)
        self.assertEqual('https://api.github.com/repos/u/r/collaborators/u',
                         collab_url)
        self.assertEqual('asdf', commit_sha)

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_with_empty_body(self, mock_policy, assem_mock,
                                          resp_mock, request_mock):
        mock_policy.return_value = True
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(400, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        assert not tw.called

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_github_webhook(self, mock_policy, assem_mock,
                                            resp_mock, request_mock):
        mock_policy.return_value = True
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'action': 'opened',
                     'pull_request': {'head': {'sha': 'asdf'}},
                     'repository': {'statuses_url': status_url}}
        expected_st_url = 'https://api.github.com/repos/u/r/statuses/asdf'
        request_mock.body = json.dumps(body_dict)
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(202, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        tw.assert_called_once_with('test_id', 'asdf', expected_st_url, None,
                                   workflow=None)

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_github_comment_webhook(self, mock_policy,
                                                    assem_mock, resp_mock,
                                                    request_mock):
        mock_policy.return_value = True
        cfg.CONF.api.rebuild_phrase = "solum retry tests"
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        collab_url = ('https://api.github.com/repos/u/r/' +
                      'collaborators{/collaborator}')
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'action': 'created',
                     'comment': {'commit_id': 'asdf',
                                 'body': '  SOLUM retry tests ',
                                 'user': {'login': 'u'}},
                     'repository': {'statuses_url': status_url,
                                    'collaborators_url': collab_url,
                                    'private': True}}
        expected_st_url = 'https://api.github.com/repos/u/r/statuses/asdf'
        request_mock.body = json.dumps(body_dict)
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(202, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        tw.assert_called_once_with('test_id', 'asdf', expected_st_url, None,
                                   workflow=None)

    @mock.patch('httplib2.Http.request')
    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_mismatch_comment_pub_repo(self, http_mock,
                                                       mock_policy,
                                                       assem_mock, resp_mock,
                                                       request_mock):
        mock_policy.return_value = True
        cfg.CONF.api.rebuild_phrase = "solum retry tests"
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        collab_url = ('https://api.github.com/repos/u/r/' +
                      'collaborators{/collaborator}')
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'action': 'created',
                     'comment': {'commit_id': 'asdf',
                                 'body': 'solum is awesome',
                                 'user': {'login': 'u'}},
                     'repository': {'statuses_url': status_url,
                                    'collaborators_url': collab_url,
                                    'private': False}}
        request_mock.body = json.dumps(body_dict)
        http_mock.return_value = ({'status': '204'}, '')  # a collaborator
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(403, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        assert not tw.called

    @mock.patch('httplib2.Http.request')
    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_valid_comment_pub_repo(self, http_mock,
                                                    mock_policy,
                                                    assem_mock, resp_mock,
                                                    request_mock):
        mock_policy.return_value = True
        cfg.CONF.api.rebuild_phrase = "solum retry tests"
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        collab_url = ('https://api.github.com/repos/u/r/' +
                      'collaborators{/collaborator}')
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'action': 'created',
                     'comment': {'commit_id': 'asdf',
                                 'body': 'solum retry tests',
                                 'user': {'login': 'u'}},
                     'repository': {'statuses_url': status_url,
                                    'collaborators_url': collab_url,
                                    'private': False}}
        expected_st_url = 'https://api.github.com/repos/u/r/statuses/asdf'
        expected_clb_url = 'https://api.github.com/repos/u/r/collaborators/u'
        request_mock.body = json.dumps(body_dict)
        http_mock.return_value = ({'status': '204'}, '')  # Valid collaborator
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(202, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        tw.assert_called_once_with('test_id', 'asdf', expected_st_url,
                                   expected_clb_url, workflow=None)

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_comment_missing_login(self, mock_policy,
                                                   assem_mock, resp_mock,
                                                   request_mock):
        mock_policy.return_value = True
        cfg.CONF.api.rebuild_phrase = "solum retry tests"
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        collab_url = ('https://api.github.com/repos/u/r/' +
                      'collaborators{/collaborator}')
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'comment': {'commit_id': 'asdf',
                                 'body': 'solum retry tests',
                                 'user': 'MISSING_LOGIN'},
                     'repository': {'statuses_url': status_url,
                                    'collaborators_url': collab_url,
                                    'private': False}}
        request_mock.body = json.dumps(body_dict)
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(400, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        assert not tw.called

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_wrong_github_webhook(self, mock_policy,
                                                  assem_mock,
                                                  resp_mock, request_mock):
        mock_policy.return_value = True
        status_url = 'https://api.github.com/repos/u/r/statuses/{sha}'
        body_dict = {'sender': {'url': 'https://api.github.com'},
                     'pull_request': {'head': {'sha': 'asdf'}},
                     'repository': {'HACKED_statuses_url': status_url}}
        request_mock.body = json.dumps(body_dict)
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(400, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        assert not tw.called

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_unknown_git_webhook(self, mock_policy, assem_mock,
                                                 resp_mock, request_mock):
        mock_policy.return_value = True
        body_dict = {"pull_request": {"head": {"sha": "asdf"}}}
        request_mock.body = json.dumps(body_dict)
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(501, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        assert not tw.called

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_non_github_webhook(self, mock_policy, assem_mock,
                                                resp_mock, request_mock):
        mock_policy.return_value = True
        body_dict = {"sender": {"url": "https://non-github.com"},
                     "pull_request": {"head": {"sha": "asdf"}}}
        request_mock.body = json.dumps(body_dict)
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(501, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        assert not tw.called

    @mock.patch('solum.common.policy.check')
    def test_trigger_post_on_github_ping_webhook(self, mock_policy, assem_mock,
                                                 resp_mock, request_mock):
        mock_policy.return_value = True
        body_dict = {"sender": {"url": "https://api.github.com"},
                     "zen": "Keep it logically awesome."}
        request_mock.body = json.dumps(body_dict)
        obj = trigger.TriggerController()
        obj.post('test_id')
        self.assertEqual(501, resp_mock.status)
        tw = assem_mock.return_value.trigger_workflow
        assert not tw.called



# Copyright 2013 - Rackspace
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_utils import uuidutils

from solum.api.handlers import handler
from solum import objects


class ComponentHandler(handler.Handler):
    """Fulfills a request on the component resource."""

    def get(self, id):
        """Return this component."""
        return objects.registry.Component.get_by_uuid(self.context, id)

    def update(self, id, data):
        """Modify a resource."""
        updated = objects.registry.Component.update_and_save(self.context,
                                                             id, data)
        return updated

    def delete(self, id):
        """Delete a resource."""
        db_obj = objects.registry.Component.get_by_uuid(self.context, id)
        db_obj.destroy(self.context)

    def create(self, data):
        """Create a new resource."""
        db_obj = objects.registry.Component()
        db_obj.uuid = uuidutils.generate_uuid()
        db_obj.update(data)
        db_obj.user_id = self.context.user
        db_obj.project_id = self.context.project_id
        db_obj.create(self.context)
        return db_obj

    def get_all(self):
        """Return all components."""
        return objects.registry.ComponentList.get_all(self.context)



# Copyright 2014 - Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from solum.objects import registry
from solum.objects.sqlalchemy import execution
from solum.objects.sqlalchemy import pipeline
from solum.tests import base
from solum.tests import utils


class TestPipeline(base.BaseTestCase):
    def setUp(self):
        super(TestPipeline, self).setUp()
        self.db = self.useFixture(utils.Database())
        self.ctx = utils.dummy_context()

        self.data = [{'project_id': self.ctx.project_id,
                      'uuid': 'ce43e347f0b0422825245b3e5f140a81cef6e65b',
                      'user_id': 'fred',
                      'name': 'pipeline1',
                      'description': 'test pipeline',
                      'trigger_id': 'trigger-uuid-1234',
                      'tags': 'pipeline tags',
                      'plan_id': 'plan_id_1'}]
        utils.create_models_from_data(pipeline.Pipeline, self.data, self.ctx)

    def test_objects_registered(self):
        self.assertTrue(registry.Pipeline)
        self.assertTrue(registry.PipelineList)

    def test_get_all(self):
        lst = pipeline.PipelineList()
        self.assertEqual(1, len(lst.get_all(self.ctx)))

    def test_check_data(self):
        ta = pipeline.Pipeline().get_by_id(self.ctx, self.data[0]['id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(ta, key))

    def test_check_data_by_trigger_id(self):
        ta = pipeline.Pipeline().get_by_trigger_id(self.ctx, self.data[0][
            'trigger_id'])
        for key, value in self.data[0].items():
            self.assertEqual(value, getattr(ta, key))

    def test_last_execution(self):
        ta = pipeline.Pipeline().get_by_id(self.ctx, self.data[0]['id'])
        # add executions
        ex1 = execution.Execution()
        ex1.uuid = 'first'
        ex1.pipeline_id = ta.id
        ex1.create(self.ctx)

        ex2 = execution.Execution()
        ex2.uuid = 'second'
        ex2.pipeline_id = ta.id
        ex2.create(self.ctx)

        extest = ta.last_execution()
        self.assertEqual('second', extest.uuid)

    def test_last_execution_none(self):
        ta = pipeline.Pipeline().get_by_id(self.ctx, self.data[0]['id'])
        extest = ta.last_execution()
        self.assertIsNone(extest)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from solum.api.controllers import common_types
from solum.api.controllers.v1.datamodel import types as api_types


class Extensions(api_types.Base):
    """CAMP v1.1 extensions resource model."""

    extension_links = [common_types.Link]
    """This attribute contains Links to extension resources that contain
     information about the extensions supported by this Platform."""

    def __init__(self, **kwds):
        super(Extensions, self).__init__(**kwds)
